/*
   This file is part of the MoCA video library.
   Name: CutDetection.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/

// C++ includes
#include <string>
#include <iostream>
using namespace std;

// MoCA includes
#include <CutDetectionFeature.h>
#include <IImageStream.h>
#include <IImageStream.h>
#include <FileList.h>
#include <FileReader.h>


void usage()
{
	cerr << "Usage: VideoStatistic [options] filename|path" << endl;
	cerr << endl;
	cerr << "options:" << endl;
	cerr << " -recurse    analyses all MPG/AVI files in path" << endl;
	cerr << endl;
	cerr << "Example:" <<  endl;
	cerr << "VideoStatistic test.mpg" << endl;
	cerr << "VideoStatistic -recurse /opt/movies/" << endl;
	exit(1);
} // usage

int main(int argc, char* argv[])
{

    long int startTime = time(NULL);
		
    if (argc<2) usage();
	string filename =argv[argc-1];

	string s;
	bool recurse=false;
	for (int i=1; i<argc; i++) {
		s=argv[i];
		for (uint32 k=0; k<s.size(); k++)
		  s[k]=toupper(s[k]);
		
		if (s=="-RECURSE") recurse=true;
		else {
			if (i != argc-1) usage();
		} // else
	} // for
	
	int32 totalSeconds = 0;
	VideoIndexData data;
    if (!recurse) {
		VideoIndex index (filename);
		index.process (data);
		totalSeconds +=  (int32)(data.videoSize[DIM_Z] / data.fps);
		cout << data << endl;
	} else {

		// read all MPEG files in directory
		vector<FileInfo> mpgFiles, aviFiles;
		FileList files(filename, true);    
		files.process("*.mpg", mpgFiles);
		files.process("*.avi", aviFiles);


		// open mpeg files
	    vector<FileInfo>::iterator it;
		for (it = mpgFiles.begin(); it != mpgFiles.end(); it++) {

			cout << endl  << "processing file: "<< (*it).name << endl;
			VideoIndex index ((*it).name);
			index.process (data);
			totalSeconds +=  (int32)(data.videoSize[DIM_Z] / data.fps);
			cout << data << endl;
		} // for

		// open avi files
		for (it = aviFiles.begin(); it != aviFiles.end(); it++) {

			cout << endl  << "processing file: "<< (*it).name << endl;
			VideoIndex index ((*it).name);
			index.process (data);
			totalSeconds +=  (int32)(data.videoSize[DIM_Z] / data.fps);
			cout << data << endl;
		} // for

		// print
		cout << endl << (aviFiles.size() + mpgFiles.size()) << " files found." << endl;

	} // else

	cerr << endl << "time for computation      : " << (time(NULL)-startTime)  << " seconds" << endl;
	cerr << endl << "total length of the videos: " << totalSeconds << " seconds" << endl;
	
	return 0;
} // main

