/*
   This file is part of the MoCA video library.
   Name: Denoise.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include <iostream>
#include <sstream>
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <DenoiseFilter.h>

int main(int argc, char* argv[])
{
  if (argc != 5) { 
    cerr << "Usage: Denoise src(pnm) dest(pnm) [theshold similar]  [theshold different]" << endl;
    return 1;
  } // if

  Image<uint8> img;

  // open input filestream and read image
  IImageStream<uint8> iis(argv[1]);
  iis.read(img);

  // read threshold parameter
  stringstream puffer1 (argv[3]);
  float threshold1=0;
  puffer1 >> threshold1;
  stringstream puffer2 (argv[4]);
  float threshold2=0;
  puffer2 >> threshold2;
  
  // manipulate image
  DenoiseFilter<uint8> df (threshold1, threshold2);
  df.process (img);

  // open output filestream and write image
  OImageStream<uint8> ois(argv[2]);
  ois.write(img);

  return 0;
} // main



