/*
   This file is part of the MoCA video library.
   Name: DownSample.cpp
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: DownSample.cpp,v 1.2 2004/03/02 17:04:11 kopf Exp $

*/

#include <vector>

#include <iostream>
#include <Video.h>
#include <IVideoStream.h>
#include <OVideoStream.h>
#include <DownSampleFilter.h>
#include <MocaExceptions.h>

int main(int argc, char* argv[])
{
    if (argc != 5) {
        cerr << "Usage: DownSample srcimage destimage numberOfPyramids directions(x,y,xy)" << endl;
        return 1;
    } // if

    string fn1=argv[1];
    string fn2=argv[2];
    int numberOfPyramids = atoi(argv[3]);
    string sdirection = argv[4];
    uint32 direction = DIR_X | DIR_Y;    
    if (sdirection == "X") direction = DIR_X;
    if (sdirection == "Y") direction = DIR_Y;    
    
    Image<uint8> src;
    ImageVector<uint8> dest (numberOfPyramids);    

    ReadImage(fn1, src);
    try {
        // manipulate image
        DownSampleFilter<uint8> filter (numberOfPyramids, direction);
        filter.process (src, dest);
    } catch (MocaException e) {
        cout << "Filter " << e.what() << endl;
        exit (-1);
    } // try

    //    Image<uint8> destImg = dest.getImage(numberOfPyramids-1);
    //WriteImage(destImg, fn2);
    
    return 0;
} // main



