/*
   This file is part of the MoCA video library.
   Name: GaussDerivation.cpp
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: GaussDerivation.cpp,v 1.1 2004/02/17 11:48:04 kopf Exp $

   
*/

#include <iostream>
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <GaussDerivationFilter.h>
#include <MocaExceptions.h>

int main(int argc, char* argv[])
{
    if (argc != 4) {
        cerr << "Usage: GaussDerivation src(pnm) dest(pnm) sigma" << endl;
        return 1;
    } // if

    // load image
    Image<uint8> img;
    try {
        // open input filestream and read image
        IImageStream<uint8> iis(argv[1]);
        iis.read(img);
    } catch (MocaException e) {
        cout << "IImageStream " << e.what() << endl;
        exit (-1);
    } // try    
        
    ieee64 sigma = atof(argv[3]);
    cout << "sigma: " << sigma << endl;
    GaussDerivationFilter<uint8> gdf (sigma, DIR_Y);
    gdf.process(img);

    
    // save image
    try {
        // open output filestream and write image
        OImageStream<uint8> ois(argv[2]);
        ois.write(img);
    } catch (MocaException e) {
        cout << "OImageStream " << e.what() << endl;
        exit (-1);
    } // try

    return 0;
} // main


