/*
   This file is part of the MoCA video library.
   Name: Jpg2MpgTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/

// C++ includes
#include <iomanip>
#include <sstream>
#include <iostream>
using namespace std;

// Moca includes
#include <Image.h>
#include <IImageStream.h>
#include <OMpegVideo.h>
#include <CropFilter.h>
#include <FileList.h>


int main(int argc, char* argv[])
{

    if ((argc != 3) && (argc != 4)) {
        cerr << "read all JPEG files in directory and store them as MPEG video" << endl << endl;
        cerr << "Usage: " << endl;
		cerr << "Jpg2Mpg srcpath dest.mpg [framerate]" << endl;
        return 1;
    } // if
	string srcPath = argv[1];
	string mpegFile = argv[2];
	double framerate = 25.0;
    if (argc == 4) {
		string tmp = argv[3];
		framerate = atof(tmp.c_str());
	} // if


	// read all JPEG files in directory
	vector<FileInfo> files;
	FileList fl(srcPath, true);    
	fl.process("*.jpg", files);

	if (files.size()==0) {
		cerr << "no files found!" << endl;
	} // if

	// get image size
	vector<FileInfo>::iterator it = files.begin();
	Image<uint8> img;
	ImageSize size;
	{
		// open image
		IImageStream<uint8> iis ((*it).name);
		iis.read(img);
		size = img.getSize();
	}



	// open writer
	bool createGrayscale=false;
	bool cropTo16=true;
	ieee64 bitrate = 1.5 * 1024 * 1024;
	OMpegVideo writer (mpegFile, size, framerate, bitrate, createGrayscale, cropTo16);


	// open files
	for (it = files.begin(); it != files.end(); it++) {

		// open image
		IImageStream<uint8> iis ((*it).name);
		iis.read(img);
		
		//cout << endl  << "write file: "<< (*it).name << endl;
		writer.write (img);
		
	} // for

	for (int32 i=0; i<5; i++) {
		writer.write (img);
	} // for

	cerr << endl;
	writer.close();

	return 0;
} // main

