/*
   This file is part of the MoCA video library.
   Name: Mpg2Yuv.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt


*/

// C++ includes
#include <iostream>
#include <sstream>
using namespace std;

// Moca includes
#include <Image.h>
#include <VideoDecoder.h>
#include <OYuvVideo.h>


int main(int argc, char* argv[])
{
    if (argc != 4) {
        cerr << "Usage: Yuv2Jpg  src(yuv)  dest(mpg)  mode" << endl;
		cerr << "values for mode: YUV, Y" << endl;
        return 1;
    } // if
	string mpegFilename = argv[1];
	string yuvFilename  = argv[2];
	string strMode = argv[3];
	VideoCodec destCodec = CODEC_YUV420;
	if ((strMode=="Y") || (strMode=="y")) destCodec = CODEC_Y;


	try {
		VideoCodec srcCodec=CODEC_RGB;
		VideoDecoder reader (mpegFilename, srcCodec);
		VideoSize size = reader.getSize();

		OYuvVideo writer(yuvFilename, srcCodec, destCodec);

		cout << "read and save images (" << 0 << "..." << size[DIM_Z]-1 << ")..." << endl;
		Image<uint8> img;
		for (int i=0; i<size[DIM_Z]; i++) {
			cerr << "frame: " << i << endl;
			img = reader.getImage(i);

			writer.write (img);
		} // for

    } catch (MocaException e) {
        cerr << "error: could not save yuv-file." << endl;
        cerr << e.what() << endl;
        exit (-1);
    } // try
	cout << endl;
	return 0;
} // main



