/*
   This file is part of the MoCA video library.
   Name: BlockMotionFeature.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   Functions for block motion vector estimation

   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: BlockMotionFeature.h,v 1.2 2004/03/02 17:04:11 kopf Exp $

*/

#ifndef BLOCKMOTIONFEATURE_H
#define BLOCKMOTIONFEATURE_H

#include <MocaTypes.h>
#include <Video.h>
#include <ScaleFilter.h>
#include <DrawLine.h>

/*!
  Filter for calculating block motion vectors. Currently only a
  very simple full search algorithm is implemented.
*/

class BlockMotionFeature
{
public:
    //! The constructor
    BlockMotionFeature(sizeType blockSize, sizeType searchRange)
            : blockSize(blockSize),
              searchRange(searchRange) 
	{
		if (searchRange>120) searchRange=120;
	}

    //! The destructor
    ~BlockMotionFeature() {}

    //! Calculate motion vectors
    void process (const Video<uint8>&, Video<int8>&);
    void process (Image<uint8>& src1, Image<uint8>& src2, Image<int8>& motionvectors);
    void processFullSearch (Image<uint8>& src1, Image<uint8>& src2, ieee64 &mx, ieee64 &my);
	void drawMotionVectors (const Image<int8>& motionVectors, Image<uint8>& srcDest);

private:
    //! Calculate absolute distance between to macro blocks
    ieee64 match(const Video<uint8>&, indexType, indexType, indexType, indexType, indexType);
    ieee64 match(Image<uint8>&, Image<uint8>&, indexType, indexType, indexType, indexType);

    //! Full search algorithm
    void fullSearch(const Video<uint8>&, Video<int8>&);
    void fullSearch(Image<uint8>&, Image<uint8>&, Image<int8>&);
	void fullSearchScaled(Image<uint8>& src1, Image<uint8>& src2, Image<int8>& defaultMotionVectors, Image<int8>& motionVectors);

    //! The size of the macroblocks
    uint8 blockSize;
    //! The search range for the full search algorithm
    int8 searchRange;
}; // BlockMotionFeature

#endif

