/*
   This file is part of the MoCA video library.
   Name: CssDatabase.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


/* -*- C++ -*-

     -- short description

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

              $Id: CssDatabase.h,v 1.1 2004/02/17 11:52:58 kopf Exp $
*/


// CSS create database
// create a new database by analysing a directory/subdirectories,
// the database is stored as header-file (*.h), and can be included in CSS_DB_Schema.h

#ifndef CSS_CREATE_DATABASE_H
#define CSS_CREATE_DATABASE_H

// MoCA includes
#include <MocaTypes.h>
#include <ImagePosition.h>
#include <ImageEdgePosition.h>
#include <CssMaximaFeature.h>
#include <FileList.h>

#include <CssMatching.h>
#include <CssDisplayResult.h>
#include <CSS_DB_Schema.h>


// C includes
#include <string>

// C++ includes
#include <string>
#include <list>
#include <set>


/*!
  \brief handles a css database
 */





// class supports: 
//   - open database (included in header file), 
//   - create new database from directory (store database as header file: CSS_DB.h)
//   - finds the best match to a CssDatabaseItem 
class CssDatabase
{
public:
    //! the constructor
    CssDatabase(CssDatabaseSchemaType schema);
    
    //! destructor
    ~CssDatabase() {};

    // creates a database C++ header-file reading all CSS files from the specified directory
    void getDatabase (vector<CssStoredDatabaseItem> &cssData, vector<string> &objectClassName)
    {
	cssData = this->cssData;
	objectClassName = this->objectClassName;
	return;
    }

    
    // creates a database C++ header-file reading all CSS files from the specified directory
    void createDatabase (const string &databaseFilename, const string &cssSourcePath);
    
    
    // distance: 0==similar, 1000=different, 2000+=very different
    void findBestMatch(const CssDatabaseItem &src, int32 &objectClassIndex, int32 &distance);
    
    
    void setCssMaximumData (const CssDatabaseItem &src, CssMaximumData &dest);
    
   
private:
    // open a database with CSS files and stores the CSS data 
    void openDatabase ();

    CssDatabaseSchemaType schema;
    string databaseFilename;
    vector<CssStoredDatabaseItem> cssData;
    vector<string> objectClassName;

}; // class CssDatabase


#endif




