/*
    CssImage.cpp     calculate a CSS image
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <CssImage.h>


/*!
	static variables
*/
const uint8 CssImage::WHITE = 255;
const uint8 CssImage::BLACK = 0;
const uint8 CssImage::GREY  = 128;



/*!
  Constructor
 */
CssImage::
CssImage (ieee64 percentNoise, uint8 gaussRadius, uint16 maxSigmaSize)
{
	this->percentNoise = percentNoise;
	this->gaussRadius  = gaussRadius;
	this->maxSigmaSize = maxSigmaSize;
} // CssImage



inline ieee64
CssImage::abs (ieee64 x)
{
    return (x<0) ? (-x) : x; 
} // abs


uint8
CssImage::
findNewValue(int x, uint8& minValue, uint8& maxValue, Signal<bool> &isConcave)
{
	uint8 value;
	if (isConcave(x)) {
		minValue++;
		value = minValue;
		if (minValue >= GREY) {
			cerr << "error in CssImage::findNewValue (minValue too large)" << endl;
			exit(1);
		} // if
	} else {
		maxValue++;
		value = maxValue;
		if (maxValue > WHITE-1) {
			cerr << "error in CssImage::findNewValue (maxValue too large)" << endl;
			exit(1);
		} // if
	} // else

	return value;
} // findNewValue


void
CssImage::
mark(int x1,int y1,int x2,int y2,int value, Image<uint8> &img, Signal<bool> &isConcave)
{
	const bool debugOn = false;
	if (debugOn) {
		cout << "("<< x1 << ","<< y1 << ")   ("<< x2 << "," << y2 << ")" << endl
			 << (int)img (x1,y1) << " "<< (int)img (x2,y2) << endl;
		if ((img (x1,y1) != BLACK) || (img (x2,y2) != BLACK)) {
			cout << "fill hole or ERROR" << endl;
		} // if

		cout << endl;
		cout << (int)img(x1-2,y1  ) << " " << (int)img(x1-1,y1  )<< " " << (int)img(x1,y1  ) << " "
             << (int)img(x1+1,y1  ) << " " << (int)img(x1+2,y1  ) << endl;
		cout << (int)img(x1-2,y1+1) << " " << (int)img(x1-1,y1+1)<< " " << (int)img(x1,y1+1) << " "
             << (int)img(x1+1,y1+1) << " " << (int)img(x1+2,y1+1) << endl;
		cout << (int)img(x1-2,y1+2) << " " << (int)img(x1-1,y1+2)<< " " << (int)img(x1,y1+2) << " "
             << (int)img(x1+1,y1+2) << " " << (int)img(x1+2,y1+2) << endl;
		cout << (int)img(x1-2,y1+3) << " " << (int)img(x1-1,y1+3)<< " " << (int)img(x1,y1+3) << " "
             << (int)img(x1+1,y1+3) << " " << (int)img(x1+2,y1+3) << endl;
		cout << (int)img(x1-2,y1+4) << " " << (int)img(x1-1,y1+4)<< " " << (int)img(x1,y1+4) << " "
             << (int)img(x1+1,y1+4) << " " << (int)img(x1+2,y1+4) << endl;
		cout << endl;
	}

	ImageSize size = img.getSize();
	bool moveLeft;
	int x,y;
	uint8 c;

	// mark left part
	moveLeft = true;
	x = x1;
	y = y1;
	while (y < size[DIM_Y])
	{
		img (x,y) = value;


	if (debugOn) {
		cout << "("<< x << ","<< y << ")"  << endl
			 << (int)img (x,y) << endl;
		if (img (x,y) != BLACK) {
			cout << "fill hole or ERROR" << endl;
		} // if

		cout << endl;
		cout << (int)img(x-2,y  ) << " " << (int)img(x-1,y  )<< " " << (int)img(x,y  ) << " "
             << (int)img(x+1,y  ) << " " << (int)img(x+2,y  ) << endl;
		cout << (int)img(x-2,y+1) << " " << (int)img(x-1,y+1)<< " " << (int)img(x,y+1) << " "
             << (int)img(x+1,y+1) << " " << (int)img(x+2,y+1) << endl;
		cout << (int)img(x-2,y+2) << " " << (int)img(x-1,y+2)<< " " << (int)img(x,y+2) << " "
             << (int)img(x+1,y+2) << " " << (int)img(x+2,y+2) << endl;
		cout << (int)img(x-2,y+3) << " " << (int)img(x-1,y+3)<< " " << (int)img(x,y+3) << " "
             << (int)img(x+1,y+3) << " " << (int)img(x+2,y+3) << endl;
		cout << (int)img(x-2,y+4) << " " << (int)img(x-1,y+4)<< " " << (int)img(x,y+4) << " "
             << (int)img(x+1,y+4) << " " << (int)img(x+2,y+4) << endl;
		cout << endl;
	}



		// count new directions
		c=0;
		if (img(x,y+1) == BLACK) {
			c++;
		} // if
		if (img(x-1,y+1) == BLACK) {
			c++;
		} // if
		if (img(x+1,y+1) == BLACK) {
			c++;
		} // if

		// no direction: draw line to bottom or next black pixel
		if (c==0) {
            if (img(x,y+1) == WHITE) {
                img (x,y+1) = BLACK;
            } else {
                if (img(x-1,y+1) == WHITE) {
                    img (x-1,y+1) = BLACK;
                } else {
                    if (img(x+1,y+1) == WHITE) {
                        img (x+1,y+1) = BLACK;
                    } else {
                        cerr << "ERROR in CssImage: mark: left part (no direction found)" << endl;
                        exit(1);
                    }
                } // else
            } // else
			y--;
		} // if

		// only one direction
		if (c==1) {
			if (img(x-1,y+1) == BLACK) {
				x--;
				moveLeft = true;
			} else {
				if (img(x+1,y+1) == BLACK) {
					x++;
					moveLeft = false;
				} // if
			} // else
		} // if

		// two directions
		if (c==2) {
			if (img(x,y+1) != BLACK) {
				if ((moveLeft) && (img(x-1,y+1) == BLACK)) {
					x--;
				} // if
				if ((!moveLeft) && (img(x+1,y+1) == BLACK)) {
					x++;
				} // if
			} // if
		} // if

		// three directions
		if (c==3) {
			if ((moveLeft) && (img(x-1,y+1) == BLACK)) {
				x--;
			} else {
				x++;
			} // else
		} // if

		y ++;
	} // while
	x1 = x;

	// mark right part
	moveLeft = false;
	x = x2;
	y = y2;
	while (y < size[DIM_Y])
	{
		img (x,y) = value;



	if (debugOn) {
		cout << "("<< x << ","<< y << ")"  << endl
			 << (int)img (x,y) << endl;
		if (img (x,y) != BLACK) {
			cout << "fill hole or ERROR" << endl;
		} // if

		cout << endl;
		cout << (int)img(x-2,y  ) << " " << (int)img(x-1,y  )<< " " << (int)img(x,y  ) << " "
             << (int)img(x+1,y  ) << " " << (int)img(x+2,y  ) << endl;
		cout << (int)img(x-2,y+1) << " " << (int)img(x-1,y+1)<< " " << (int)img(x,y+1) << " "
             << (int)img(x+1,y+1) << " " << (int)img(x+2,y+1) << endl;
		cout << (int)img(x-2,y+2) << " " << (int)img(x-1,y+2)<< " " << (int)img(x,y+2) << " "
             << (int)img(x+1,y+2) << " " << (int)img(x+2,y+2) << endl;
		cout << (int)img(x-2,y+3) << " " << (int)img(x-1,y+3)<< " " << (int)img(x,y+3) << " "
             << (int)img(x+1,y+3) << " " << (int)img(x+2,y+3) << endl;
		cout << (int)img(x-2,y+4) << " " << (int)img(x-1,y+4)<< " " << (int)img(x,y+4) << " "
             << (int)img(x+1,y+4) << " " << (int)img(x+2,y+4) << endl;
		cout << endl;
	}



		// count new directions
		c=0;
		if (img(x,y+1) == BLACK) {
			c++;
		} // if
		if (img(x-1,y+1) == BLACK) {
			c++;
		} // if
		if (img(x+1,y+1) == BLACK) {
			c++;
		} // if

		// no direction: draw line to bottom or next black pixel
		if (c==0) {
            if (img(x,y+1) == WHITE) {
                img (x,y+1) = BLACK;
            } else {
                if (img(x+1,y+1) == WHITE) {
                    img (x+1,y+1) = BLACK;
                } else {
                    if (img(x-1,y+1) == WHITE) {
                        img (x-1,y+1) = BLACK;
                    } else {
                        cerr << "ERROR in CssImage: mark: right part (no direction found)" << endl;
                        exit(1);
                    }
                } // else
            } // else
			y--;
		} // if

		// only one direction
		if (c==1) {
			if (img(x+1,y+1) == BLACK) {
				x++;
				moveLeft = false;
			} else {
				if (img(x-1,y+1) == BLACK) {
					x--;
					moveLeft = true;
				} // if
			} // else
		} // if

		// two directions
		if (c==2) {
			if (img(x,y+1) != BLACK) {
				if ((moveLeft) && (img(x-1,y+1) == BLACK)) {
					x--;
				} // if
				if ((!moveLeft) && (img(x+1,y+1) == BLACK)) {
					x++;
				} // if
			} // if
		} // if

		// three directions
		if (c==3) {
			if ((moveLeft) && (img(x-1,y+1) == BLACK)) {
				x--;
			} else {
				x++;
			} // else
		} // if

		y ++;
	} // while
	x2 = x;

	// mark concave and convex parts
	for (x=x1; x<=x2; x++) {
		isConcave(x) = !isConcave(x);
	} // for

	/*
	for (x=0; x<size[DIM_X]; x++) {
		if (isConcave(x)) {
			cout << "x";
		} else {
			cout << ".";
		} // else
	} // for
	cout << endl;
	*/

	return;
} // mark


void
CssImage::
fillHoles(Image<uint8> &img)
{
	int x, y, i;
	ImageSize size = img.getSize();

	// save convex and concave parts
	SignalSize tmpSignalSize (size[DIM_X]);
	Signal<bool> isConcave(tmpSignalSize, SignalSize (0), BORDER_PERIODIC);
	for (x=0; x<size[DIM_X]; x++) {
		isConcave(x) = true;
	} // for
	uint8 minC = BLACK;
	uint8 maxC = GREY;
	uint8 newValue;
	const uint8 maxWidth = 3;

	// set bottom line white
	double nrBottomLines = ((double) size[DIM_Y] * percentNoise) / 100.0;
	if (nrBottomLines < 1) {
		nrBottomLines = 1;
	} // if
	//cout << "nrBottomLines:"<< nrBottomLines << endl;
	//cout << "size:"<< size[DIM_Y] << endl;
	//cout << "percentNoise:"<< (int)percentNoise << endl;

	for (y=0; y<nrBottomLines; y++) {
		for (x=0; x<size[DIM_X]; x++) {
			img(x,size[DIM_Y]-1-y) = WHITE;
		} // for
	} // for

	// process image
	for (y=0; y<size[DIM_Y]-nrBottomLines; y++) {


	/*for (int ee=0; ee<10; ee++) {
	for (int dd=0; dd<10; dd++) {
	int ff = img (x-5+dd, y-5+ee);
	if (ff<10)  cout << " ";
	if (ff<100) cout << " ";
	cout << ff << " ";
	} // for
	cout << endl;
	} // for
	cout << endl;
		*/


		// (1)  find normal css parts (without holes)
		for (x=0; x<size[DIM_X]; x++) {
			if (img(x,y) == BLACK) {

				// count black pixel
				int blackPixel = 0;
				for (i=x-1; i<=x+1; i++) {
					if (img(i,y) == BLACK) {
						blackPixel++;
					} // if
					if (img(i,y+1) == BLACK) {
						blackPixel++;
					} // if
				} // for

				if (blackPixel > 2) {
					int xLeft  = x;
					int yLeft  = y;
					int xRight = x;
					int yRight = y;
					if (img(x+1,y) == BLACK) {
						xRight = x+1;
					} else {
						if (img(x-1,y+1) == BLACK) {
							xLeft = x-1;
							yLeft = y+1;
						} else {
							xRight = x+1;
							yRight = y+1;
						} // else
					} // else

					//cout << (int)img (xLeft,yLeft) << " "<< (int)img (xRight,yRight) << endl;
					// mark visited
					newValue = findNewValue ((xLeft+xRight)/2, minC, maxC, isConcave);
					mark(xLeft,yLeft, xRight, yRight, newValue, img, isConcave);
				} // if
			} // if
		} // for


		// (2)  find holes in part of css image (not topPart)
		for (x=0; x<size[DIM_X]; x++) {
			if (img(x,y) == BLACK) {

				// search line
				uint8 posX = 1;
				while ((img(x+posX,y) == WHITE) && (posX < maxWidth)) {
					posX++;
				} // white

				if ((posX>1) && (img(x+posX,y) == BLACK)) {
					// mark visited
					newValue = findNewValue (x+(posX/2), minC, maxC, isConcave);

					// draw line
					for (i=x; i<=x+posX; i++) {
						img(i,y) = newValue;
						if (img(i,y-1) == BLACK) {
							img(i,y-1) = newValue;
							if (img(i,y-2) == BLACK) {
								img(i,y-2) = newValue;
							} // if
						} // if
					} // for
					mark(x,y, x+posX,y, newValue, img, isConcave);
				} // if
			} // if
		} // for

	} // for

	// remove black pixel
	for (y=0; y<size[DIM_Y]; y++) {
		for (x=0; x<size[DIM_X]; x++) {
			if (img(x,y) == BLACK) {
				img(x,y) = WHITE;
			} // if
		} // for
	} // for

	// remove pixel in top line
	for (x=0; x<size[DIM_X]; x++) {
		img(x,0) = WHITE;
	} // for

	return;
} // fillHoles



/*!
  main routine to generate a CSS image
 */
void
CssImage::
process(const vector<ImagePosition> &src, Image<uint8> &img)
{
#if 0
		cerr << "debug: reduze size of CSS image" << endl;
		maxSigmaSize = 850;
#endif


	uint16 nx = src.size();
	SignalSize tmpSignalSize (nx);

    Signal<ieee64> contourX  (tmpSignalSize, SignalSize (0), BORDER_PERIODIC);
    Signal<ieee64> contourY  (tmpSignalSize, SignalSize (0), BORDER_PERIODIC);
    Signal<ieee64> curvature (tmpSignalSize, SignalSize (0), BORDER_PERIODIC);

	// copy src to contour
	int i;
	vector<ImagePosition>::const_iterator it = src.begin();
	for (i=0; i< nx; i++) {
		contourX (i) = (*it)[DIM_X];
		contourY (i) = (*it)[DIM_Y];
		it++;
	} // for

	// initialize css image
	Image<uint8> tempImg(ImageSize (nx,maxSigmaSize,1), ImageSize (0,0,0), BORDER_PERIODIC);
	int j;
	for (i=0; i<nx; i++) {
		for (j=0; j<maxSigmaSize; j++) {
			tempImg(i,j) = WHITE;
		} // for
	} // for
	for (i=0; i<nx; i++) {
		tempImg(i,0) = BLACK;
	} // for

    // calculate css-image
    // calculate css-image
    int32 gaussIteration = 0;
	ieee64 gaussIterationPrecise = 0.0;
	ieee64 gaussIterationStep = 1.0;

    GaussFilter<ieee64> gauss (2*gaussRadius + 1);



#if 0
	// debug: store smoothed contours
	bool maxGaussIteration=false;

	// store smoothed contours
	// find minX, minY, maxX, maxY
	cerr << "CssImage::store many objects to D:/temp/" << endl;
	int32 minX=10000;
	int32 minY=10000;
	int32 maxX=-1;
	int32 maxY=-1;
	for (i=0; i< nx; i++) {
		if (minX>contourX (i)) minX=contourX (i);
		if (minY>contourY (i)) minY=contourY (i);
		if (maxX<contourX (i)) maxX=contourX (i);
		if (maxY<contourY (i)) maxY=contourY (i);
	} // for

	// initialize image
	int borderSize=8;
	ImageSize tmpStoreSize (maxX-minX+2*borderSize,maxY-minY+2*borderSize,3);
	Image<uint8> tmpStore(tmpStoreSize);
#endif

	int objectSmoothed = 0;
    while ((objectSmoothed<3) || (!(gaussIteration > 30))) { // do at least 30 iterations

#if 0
		// debug: store smoothed contours
		{
			// do not store all frames, from frame 100+, store every second frame
			int32 modFactor=1 + gaussIteration / 100;
			if (gaussIteration % modFactor == 0) {
				

				// clear image
				tmpStore.clear(255);
				
				// draw lines
				DrawLine line;
				for (i=0; i< nx; i++) {
					ImagePosition p1(contourX (i)-minX+borderSize, contourY (i)-minY+borderSize);
					ImagePosition p2(contourX (i+1)-minX+borderSize, contourY (i+1)-minY+borderSize);
					line.process(p1,p2,Color::BLACK, tmpStore);
				} // for
				
				// set filename
				string gaussIterationString = FileHandling::getNumberedFilename ("D:/temp/contour" ,gaussIteration, ".png");
				
				Signal<ieee64> tmpCurvature (tmpSignalSize, SignalSize (0), BORDER_PERIODIC);
				
				for (i=0; i< nx; i++) {
					tmpCurvature (i+1,2) =
						( contourX(i+1,0) - contourX(i,0) ) * ( contourY(i+2,1) - contourY(i,1  ) ) +
						( contourY(i+1,1) - contourY(i,1) ) * ( contourX(i  ,0) - contourX(i+2,0) );
				} // for
				
#if 0
				// debug: mark zero crossings
				DrawDot dot(3);
				for (i=0; i< nx; i++) {
					if ( (tmpCurvature(i) * tmpCurvature(i+1) ) < 0) {
						int32 p=0;
						if ( CssImage::abs(tmpCurvature(i)) > CssImage::abs(tmpCurvature(i+1)) ) {
							p=i+1;
						} else {
							p=i;
						} // else
						ImagePosition p3(contourX (p)-minX+borderSize, contourY (p)-minY+borderSize);
						dot.process(p3,Color::RED, tmpStore);
					} // if
				} // for
#endif				
				// store file
				OImageStream<uint8> ois(gaussIterationString);
				ois.write(tmpStore);
			} // if: modFactor
		}
#endif


		gaussIterationPrecise += gaussIterationStep;
		gaussIteration = (int32)gaussIterationPrecise;

		if (gaussIteration >= 0.5*maxSigmaSize-2) {
			gaussIterationStep = 0.6;
			if (gaussIteration >= 0.65*maxSigmaSize-2) {
				gaussIterationStep = 0.4;
				if (gaussIteration >= 0.75*maxSigmaSize-2) {
					gaussIterationStep = 0.3;
					if (gaussIteration >= 0.85*maxSigmaSize-2) {
						gaussIterationStep = 0.2;
						if (gaussIteration >= 0.95*maxSigmaSize-2) {
							gaussIterationStep = 0.1;
							if (gaussIteration >= maxSigmaSize-2) {
								gaussIterationStep = 0.0;
							} // if
						} // if
					} // if
				} // if
			} // if
		} // if



/*
		gaussIteration++;
		if (gaussIteration >= maxSigmaSize-1) {
			if (!maxGaussIteration) {
				cerr << "error in CssImage::process (gaussIteration > " << maxSigmaSize-2 << ")" << endl;
				cerr << "check, if parametrized contour is valid" << endl << endl;
				maxGaussIteration = true;
			} // if

			gaussIteration -= 1;
			//exit(1);
		} // if
*/

		// curvature evolution
	    gauss.process (contourX);
	    gauss.process (contourY);

		// sign of curvature
		for (i=0; i< nx; i++) {
			curvature (i+1) =
				( contourX(i+1) - contourX(i) ) * ( contourY(i+2) - contourY(i  ) ) +
				( contourY(i+1) - contourY(i) ) * ( contourX(i  ) - contourX(i+2) );
		} // for

		// mark zero crossings
		objectSmoothed++;
		for (i=0; i< nx; i++) {
            if ( (curvature(i) * curvature(i+1) ) < 0) {
				objectSmoothed = 0;
                if ( CssImage::abs(curvature(i)) > CssImage::abs(curvature(i+1)) ) {
                    tempImg (i+1,gaussIteration) = BLACK;
                } else {
                    tempImg (i,gaussIteration) = BLACK;
                } // else
            } // if
        } // for

	} // while

	// copy tempImg to img (flip image)
	img.realloc(ImageSize (nx,gaussIteration+2,1), ImageSize (0,0,0), BORDER_PERIODIC);
	for (i=0; i<nx; i++) {
		img(i,0)   = WHITE;
		img(i,1)   = WHITE;
		for (j=0;j<gaussIteration; j++) {
			img(i,j+2) = tempImg(i,gaussIteration-j);
		} // for
	} // for

	// fill holes
	fillHoles(img);

	return;
} // process



/*!
  Process
	src is a signal<int16> with band==0: x-coordinates of the image, and
	                            band==1: y-coordinates of the image
 */
void
CssImage::
process(const Signal<int16> &src, Image<uint8> &img)
{
	// copy data to vector
	int32 nx = src.getSize()[DIM_X];
	vector<ImagePosition> srcList;
	for (int32 i=0; i<nx; i++) {
		ImagePosition p(src(i,0),src(i,1));
		srcList.push_back(p);
	} // for

	// call process with vector as parameter
	process(srcList, img);

	return;
} // process



// caluclate parametrized contour
// input: image with an object
// output: parametrized contour
void 
CssImage::getContour(const Image<uint8> &img, vector<ImagePosition> &contour) 
{
	// get contour pixels
	vector<ImageEdgePosition> iepList;
	ContourEdgeList cel;
	cel.process (img, iepList);

	// parametrize contour pixels, the contour is described with a fixed number of pixels (DEFAULT_CONTOUR_SIZE)
	contour.clear();
	ContourParametrization cp(ContourParametrization::DEFAULT_CONTOUR_SIZE);
	cp.process(iepList, contour);


#if 0
	// debug: create an image with the detected and parametrized contour pixels
	ImageSize size = img.getSize();
	size[DIM_BAND] = 1;
	Image<uint8> tmp(size);
	tmp.clear(255);
	DrawDot dot(1);
	vector<ImagePosition>::iterator ipIt;
	for (ipIt= contour.begin(); ipIt != contour.end(); ipIt++) {
		dot.process((*ipIt), 0, tmp);
	} // for
	WriteImage (tmp, "D:/images/contour-debug.png");
#endif


	return;
}


// main routine that generates a CSS image
//   src: input image
//   img: CSS image
void
CssImage::
process(const Image<uint8> &src, Image<uint8> &img)
{
	// get contour pixels
	vector<ImagePosition> ipList;
	getContour(src, ipList);

	
	// calc css image
	process(ipList, img);

	return;
} // process




void 
CssImage::calcInvertedContour (Signal<int16> &ip)
{
	// (1) get center of object
	SignalSize size = ip.getSize();
	ieee64 cX=0.0;
	ieee64 cY=0.0;
	int32 n=0;
	for (int32 i=0; i<size[DIM_X];i++) {
		cX += ip(i,0);
		cY += ip(i,1);
		n++;
	} // for
	if (n==0) n=1;
	cX /= ((ieee64)n); 
	cY /= ((ieee64)n); 
	centerX = (int32)(cX+0.5);
	centerY = (int32)(cY+0.5);



	// (2) check, if center is within original contour
	//            TODO: improve speed

	// create contour image 
	int32 minX=10000;
	int32 maxX=0;
	int32 minY=10000;
	int32 maxY=0;
	for (int32 i=0; i<size[DIM_X];i++) {
		if (minX>ip(i,0)) minX=ip(i,0);
		if (maxX<ip(i,0)) maxX=ip(i,0);
		if (minY>ip(i,1)) minY=ip(i,1);
		if (maxY<ip(i,1)) maxY=ip(i,1);
	} // for
	ImageSize sizeTmp (maxX-minX+6,maxY-minY+6,1);
	Image<uint8> tmp(sizeTmp);
	tmp.clear(255);
	DrawLine line(Color::BLACK);
	for (int32 i=0; i<size[DIM_X];i++) {
		ImagePosition p0 (3+ip(i,0)  - minX, 3+ip(i,1)  -minY);
		ImagePosition p1 (3+ip(i+1,0)- minX, 3+ip(i+1,1)-minY);
		line.process(p0,p1,tmp);
	}
	//WriteImage(tmp,"D:/contour.png");


	// locate largest object (not background) and dilate (shrink) object
	Image<uint8> object;
	{
		RegionFeature<uint8> rf;
		bool objectFound;
		rf.process(tmp, object, objectFound);

		DilationFilter<uint8> dilate(5);
		dilate.process (object);
	}
	//WriteImage(object,"D:/object.png");

	// update position of center
	int32 tmpX = centerX+3-minX;
	int32 tmpY = centerY+3-minY;

	//DrawCross cross(6,Color::GRAY);
	//ImagePosition p0 (tmpX,tmpY);
	//cross.process(p0,tmp);
	//WriteImage(tmp,"D:/center.png");

	// check if center is within object (if not, correct the center)
	if (object(tmpX,tmpY) != 0) {
		bool centerOkay = false;
		int32 radius = 0;
		while (!centerOkay) {
			radius++;
			for (int32 y=tmpY-radius; y<=tmpY+radius; y++) {
				for (int32 x=tmpX-radius; x<=tmpX+radius; x++) {
					if (object(x,y) == 0) {
						centerOkay = true;
						tmpX=x;
						tmpY=y;
						break;
					} // if
				} // for
				if (centerOkay) break;
			} // for
		} // while
	} // if
	//ImagePosition p1 (tmpX,tmpY);
	//cross.process(p1,tmp);
	//WriteImage(tmp,"D:/center2.png");

	// set center
	centerX = tmpX-3+minX;
	centerY = tmpY-3+minY;


	// (3) get distance to center for each contour pixel and create a virtual radius
	SignalSize distSize (size[DIM_X]);
	Signal<ieee64> dist (distSize);
	dist.clear(0);
	n=0;
	radius = 0; // maximum distance
	for (int32 i=0; i<size[DIM_X];i++) {
		ieee64 d1 = centerX-ip(i,0);
		ieee64 d2 = centerY-ip(i,1);
		dist(n) = sqrt(d1*d1+d2*d2);
		if (radius < dist(n)) radius = (int32)dist(n);
		n++;
	} // for
	radius +=3;


	// (4) invert original contour
	n=0;
	for (int32 i=0; i<size[DIM_X];i++) {
		ieee64 factor = (radius+radius-dist(n))/dist(n);
		ieee64 vx = ip(i,0)-centerX;
		ieee64 vy = ip(i,1)-centerY;
		vx *= factor;
		vy *= factor;
		vx += centerX;
		vy += centerY;

		ip(i,0) = (int32)(vx+0.5);
		ip(i,1) = (int32)(vy+0.5);
		n++;
	} // for


	return;
}


void 
CssImage::resampleInvertedContour (Signal<int16> &ip)
{

	// remove negative values
	int32 n = ip.getSize()[DIM_X];
	int32 minX = 0;
	int32 minY = 0;
	for (int32 i=0; i<n; i++) {
		if (minX>ip(i,0)) minX=ip(i,0);
		if (minY>ip(i,1)) minY=ip(i,1);
	} // for
	for (int32 i=0; i<n; i++) {
		ip(i,0) -= minX;
		ip(i,1) -= minY;
	} // for

	// get maximum value for image size
	int32 maxX = 0;
	int32 maxY = 0;
	for (int32 i=0; i<n; i++) {
		if (maxX<ip(i,0)) maxX=ip(i,0);
		if (maxY<ip(i,1)) maxY=ip(i,1);
	} // for

	ImageSize size (maxX+6,maxY+6);
	Image<uint8> img (size);
	img.clear(255);
	DrawLine line(Color::BLACK);


	// shift contour pixel by three pixels
	for (int32 i=0; i<n; i++) {
		ip(i,0) += 3;
		ip(i,1) += 3;
	} // for

	// draw contour
	ImagePosition p0 (ip(0,0),   ip(0,1));
	ImagePosition p1 (ip(n-1,0), ip(n-1,1));
	line.process(p0,p1, img);
	for (int32 i=0; i<n; i++) {
		ImagePosition p0 (ip(i,0),   ip(i,1));
		ImagePosition p1 (ip(i+1,0), ip(i+1,1));
		line.process(p0,p1, img);
	} // for

	// get contour
	vector<ImagePosition> ipList;
	getContour(img, ipList);


	// undo shift
	for (int32 i=0; i<ipList.size(); i++) {
		ipList[i][DIM_X] -=3;
		ipList[i][DIM_Y] -=3;
		ip(i,0) -=3;
		ip(i,1) -=3;
	} // for

	// undo negative values
	for (int32 i=0; i<ipList.size(); i++) {
		ipList[i][DIM_X] +=minX;
		ipList[i][DIM_Y] +=minY;
		ip(i,0) +=minX;
		ip(i,1) +=minY;
	} // for



	// find best start position (rotate one contour and get distance to the start pixel)
	int32 shift = 0;
	int32 minDist = 100000000;
	for (int32 i=0; i<ipList.size(); i++) {
		int32 px0 = ipList[i][DIM_X];
		int32 px1 = ip(0,0);
		int32 py0 = ipList[i][DIM_Y];
		int32 py1 = ip(0,1);
		
		int32 dx = (int32)abs(px0-px1);  
		int32 dy = (int32)abs(py0-py1);  
		int32 d  = dx*dx+dy*dy;
		
		if (minDist>d) {
			minDist = d;
			shift = i;
		} // if
	} // for

	// check, if contour is mirrored
	int32 px0 = ipList[(n+10+shift)%n][DIM_X];
	int32 px1 = ip(10,0);
	int32 py0 = ipList[(n+10+shift)%n][DIM_Y];
	int32 py1 = ip(10,1);
	int32 dx  = (int32)abs(px0-px1);  
	int32 dy  = (int32)abs(py0-py1);  
	int32 dStd= dx*dx+dy*dy;

	px0 = ipList[(n+10+shift)%n][DIM_X];
	px1 = ip(n-10,0);
	py0 = ipList[(n+10+shift)%n][DIM_Y];
	py1 = ip(n-10,1);
	dx  = (int32)abs(px0-px1);  
	dy  = (int32)abs(py0-py1);  
	int32 dMir= dx*dx+dy*dy;


	// copy values
	if (dStd < dMir) {
		for (int32 i=0; i<n; i++) {
			ip(i,0) = ipList[(n+i+shift)%n][DIM_X];
			ip(i,1) = ipList[(n+i+shift)%n][DIM_Y];
		} // for
	} else {
		for (int32 i=0; i<n; i++) {
			ip((n-i)%n,0) = ipList[(n+i+shift)%n][DIM_X];
			ip((n-i)%n,1) = ipList[(n+i+shift)%n][DIM_Y];
		} // for
	} // else


	return;
}





// main routine that generates a CSS image
//   src: definition of the contour of the object (vector or "signal" of contour points) 
//   stdCss: standard CSS image
//   invCss: inverted CSS image
void
CssImage::
process(const vector<ImagePosition> &src, Image<uint8> &stdCss, Image<uint8> &invCss, Image<uint8> &debugContour)
{
	// calc standard css image
	process(src, stdCss);
	
	// invert contour (switch convex and concave parts)
	SignalSize size (src.size(),2);
	Signal<int16> ip(size, SignalSize(0,0), BORDER_PERIODIC);
	vector<ImagePosition>::const_iterator it = src.begin();
	for (int32 i=0; i<size[DIM_X]; i++) {
		ip(i,0) = (*it)[DIM_X];
		ip(i,1) = (*it)[DIM_Y];
		it++;
	} // for
	calcInvertedContour (ip);
	
	// resample inverted contour
	resampleInvertedContour (ip);	


	// calc extended css image
	process(ip, invCss);


	// calculate debug contour
	// get offset 
	int32 maxX=0;
	int32 maxY=0;
	int32 minX=1000000;
	int32 minY=1000000;
	for (int32 i=0; i<size[DIM_X]; i++) {
		if (maxX< ip(i,0)) maxX=ip(i,0);
		if (maxY< ip(i,1)) maxY=ip(i,1);

		if (minX> ip(i,0)) minX=ip(i,0);
		if (minY> ip(i,1)) minY=ip(i,1);
	} // for
	int32 width  = maxX-minX + 4;
	int32 height = maxY-minY + 4;
	int32 oX = -minX+2;
	int32 oY = -minY+2;
	ImageSize imgSize (width, height,1);
	debugContour.resize (imgSize);
	debugContour.clear(255);

	// draw new object
	ImagePosition p0,p1;
	DrawLine line(1);
	for (int32 i=0; i<size[DIM_X]; i++) {
		int32 x0 = ip(i,0) + oX;
		int32 x1 = ip(i+1,0) + oX;
		int32 y0 = ip(i,1) + oY;
		int32 y1 = ip(i+1,1) + oY;
		ImagePosition p0(x0,y0);
		ImagePosition p1(x1,y1);
		line.process (p0,p1,debugContour);
	} // for


	// draw center
	ImagePosition p2(centerX+oX,centerY+oY);
	DrawCross dc;
	dc.process (p2, Color::BLACK, debugContour);

	// draw circle
	DrawCircle circle;
	circle.process (p2, radius, Color::BLACK, debugContour);
	//WriteImage (debugContour, "D:/tmp3.png");


	// print old contour
	vector<ImagePosition>::const_iterator ipIt2 = src.begin();
	vector<ImagePosition>::const_iterator ipIt1 = src.end();
	ipIt1--;

	// draw last line segment
	p0 = (*ipIt2);
	p0[DIM_X]+=oX;
	p0[DIM_Y]+=oY;
	p1 = (*ipIt1);
	p1[DIM_X]+=oX;
	p1[DIM_Y]+=oY;
	line.process(p0, p1, 0, debugContour);

	// draw contour
	for (ipIt2= src.begin(); ipIt2 != src.end(); ipIt2++) {
		p0 = (*ipIt2);
		p0[DIM_X]+=oX;
		p0[DIM_Y]+=oY;

		ipIt1 = ipIt2;
		if (ipIt1 != src.end()) ipIt1++;
		if (ipIt1 != src.end()) {
			p1 = (*ipIt1);
			p1[DIM_X]+=oX;
			p1[DIM_Y]+=oY;
			line.process(p0, p1, 0, debugContour);
		} // if
	} // for
	//WriteImage (debugContour, "D:/tmp4.png");
	
	return;
}



// main routine that generates two CSS images, the standard image and an "inverted" image
//   src: input image
//   stdCss: standard CSS image
//   invCss: inverted CSS image
void
CssImage::
process(const Image<uint8> &src, Image<uint8> &stdCss, Image<uint8> &invCss, Image<uint8> &debugContour)
{
	// get contour pixels
	vector<ImagePosition> ipList;
	getContour(src, ipList);

	// caculate
	process(ipList, stdCss, invCss, debugContour);

	return;
} // process


