/*
    CutDetection.cpp     detect cuts (shot boundaries) in videos
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <CutDetectionFeature.h>


std::ostream& operator<< (ostream &os, const CutDetectionData &data) 
{
	os << "version                      : " << data.version << endl;
	os << "filename                     : " << data.filename << endl;
	os << "number of hard cuts          : " << data.numberOfHardCuts << endl;
	os << "number of fade outs          : " << data.numberOfFadeOut << endl;
	os << "number of fade ins           : " << data.numberOfFadeIn << endl;
	os << "number of dissolves          : " << data.numberOfDissolves << endl;
	os << "number of wipes              : " << data.numberOfWipes << endl;
	os << "number of other scene changes: " << data.numberOfUnknownSceneChanges<< endl;
	os << "threshold                    : " << data.threshold << endl;
	os << "hardcut threshold            : " << data.thHardCut << endl;
	os << "number of frames             : " << data.numberOfFrames << endl;
	os << endl;
	
#if 0
	if (data.numberOfHardCuts > 0) {
		os << "hard cuts:" << endl;
		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_HARD_CUT) {
				if (i<10) os << " ";
				if (i<100) os << " ";
				if (i<1000) os << " ";
				if (i<10000) os << " ";
				os << i << endl;
			} // if
		} // for
	} // if

	if (data.numberOfFadeOut > 0) {
		os << endl << "fade out :" << endl;
		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_FADE_OUT) {
				int32 start = i;
				while ((i<data.cuts.getSize()[DIM_X]) && (data.cuts(i) == SHOT_CHANGE_FADE_OUT)) {
					i++;
				} // while
				i--;
				if (start<10) os << " ";
				if (start<100) os << " ";
				if (start<1000) os << " ";
				if (start<10000) os << " ";
				os << start << "..";
				if (i<10) os << " ";
				if (i<100) os << " ";
				if (i<1000) os << " ";
				if (i<10000) os << " ";
				os << i << endl;
			} // if
		} // for
	} // if

	if (data.numberOfFadeIn > 0) {
		os << endl << "fade in  :" << endl;
		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_FADE_IN) {
				int32 start = i;
				while ((i<data.cuts.getSize()[DIM_X]) && (data.cuts(i) == SHOT_CHANGE_FADE_IN)) {
					i++;
				} // while
				i--;
				if (start<10) os << " ";
				if (start<100) os << " ";
				if (start<1000) os << " ";
				if (start<10000) os << " ";
				os << start << "..";
				if (i<10) os << " ";
				if (i<100) os << " ";
				if (i<1000) os << " ";
				if (i<10000) os << " ";
				os << i << endl;
			} // if
		} // for
	} // if

	if (data.numberOfDissolves > 0) {
		os << endl << "dissolve :" << endl;
		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if ((data.cuts(i) == SHOT_CHANGE_DISSOLVE) ||
				(data.cuts(i) == SHOT_CHANGE_DISSOLVE_FIRST) ||
				(data.cuts(i) == SHOT_CHANGE_DISSOLVE_LAST) ){
				int32 start = i;
				while ((i<data.cuts.getSize()[DIM_X]) && 
					((data.cuts(i) == SHOT_CHANGE_DISSOLVE) ||
					(data.cuts(i) == SHOT_CHANGE_DISSOLVE_FIRST) ||
					(data.cuts(i) == SHOT_CHANGE_DISSOLVE_LAST))) {
					i++;
				} // while
				i--;
				if (start<10) os << " ";
				if (start<100) os << " ";
				if (start<1000) os << " ";
				if (start<10000) os << " ";
				os << start << "..";
				if (i<10) os << " ";
				if (i<100) os << " ";
				if (i<1000) os << " ";
				if (i<10000) os << " ";
				os << i << endl;
			} // if
		} // for
	} // if

	if (data.numberOfWipes > 0) {
		os << endl << "wipe:" << endl;
		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if ((data.cuts(i) == SHOT_CHANGE_WIPE) ||
				(data.cuts(i) == SHOT_CHANGE_WIPE_FIRST) ||
				(data.cuts(i) == SHOT_CHANGE_WIPE_LAST) ){
				int32 start = i;
				while ((i<data.cuts.getSize()[DIM_X]) && 
					((data.cuts(i) == SHOT_CHANGE_WIPE) ||
					(data.cuts(i) == SHOT_CHANGE_WIPE_FIRST) ||
					(data.cuts(i) == SHOT_CHANGE_WIPE_LAST))) {
					i++;
				} // while
				i--;
				if (start<10) os << " ";
				if (start<100) os << " ";
				if (start<1000) os << " ";
				if (start<10000) os << " ";
				os << start << "..";
				if (i<10) os << " ";
				if (i<100) os << " ";
				if (i<1000) os << " ";
				if (i<10000) os << " ";
				os << i << endl;
			} // if
		} // for
	} // if


	if (data.numberOfUnknownSceneChanges > 0) {
		os << "unknown scene change:" << endl;
		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_OTHER) {
				if (i<10) os << " ";
				if (i<100) os << " ";
				if (i<1000) os << " ";
				if (i<10000) os << " ";
				os << i << endl;
			} // if
		} // for
	} // if
#endif

	return os;
} // operator <<

OTagStream& operator<< (OTagStream& os, const CutDetectionData &data) 
{
    TagFormat::writeStartTag (os, "SCENE_CHANGE");
	os << endl;
	
    TagFormat::writeNodeValue(os, "version", data.version, 1);
    TagFormat::writeNodeValue(os, "filename", data.filename, 1);
	os << endl;

	TagFormat::writeStartTag (os, "numberHardCuts", 1);
	os << setfill(' ') << setw(5) << data.numberOfHardCuts; 
	TagFormat::writeEndTag (os, "numberHardCuts");

	TagFormat::writeStartTag (os, "numberFadeIn", 1);
	os << setfill(' ') << setw(7) << data.numberOfFadeIn; 
	TagFormat::writeEndTag (os, "numberFadeIn");

	TagFormat::writeStartTag (os, "numberFadeOut", 1);
	os << setfill(' ') << setw(6) << data.numberOfFadeOut; 
	TagFormat::writeEndTag (os, "numberFadeOut");

	TagFormat::writeStartTag (os, "numberDissolves", 1);
	os << setfill(' ') << setw(4) << data.numberOfDissolves; 
	TagFormat::writeEndTag (os, "numberDissolves");

	TagFormat::writeStartTag (os, "numberWipes", 1);
	os << setfill(' ') << setw(4) << data.numberOfWipes; 
	TagFormat::writeEndTag (os, "numberWipes");

	TagFormat::writeStartTag (os, "numberOther", 1);
	os << setfill(' ') << setw(8) << data.numberOfUnknownSceneChanges; 
	TagFormat::writeEndTag (os, "numberOther");
	
	os << endl;
    TagFormat::writeNodeValue(os, "threshold", data.threshold, 1);
    TagFormat::writeNodeValue(os, "thresholdHardcut", data.thHardCut, 1);
    TagFormat::writeNodeValue(os, "numberOfFrames", data.numberOfFrames, 1);
	os << endl;
	
	// write hard cuts
	if (data.numberOfHardCuts > 0) {
		TagFormat::writeStartTag (os, "HARD_CUT", 1);
		os << endl;

		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_HARD_CUT) {

				TagFormat::writeStartTag (os, "frame", 2);
				os << setfill(' ') << setw(6) << i; 
				TagFormat::writeEndTag (os, "frame");
			} // if
		} // for

		TagFormat::writeEndTag (os, "HARD_CUT", 1);
		os << endl;
	} // if

	
	// write fades
	if (data.numberOfFadeIn > 0) {
		TagFormat::writeStartTag (os, "FADE_IN", 1);
		os << endl;

		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_FADE_IN) {
				int32 left=i;
				while ((i<data.cuts.getSize()[DIM_X]-1) && (data.cuts(i+1) == SHOT_CHANGE_FADE_IN)) i++;

				TagFormat::writeStartTag (os, "firstframe", 2);
				os << setfill(' ') << setw(6) << left; 
				TagFormat::writeEndTag (os, "firstframe", -1);

				TagFormat::writeStartTag (os, "lastframe", 2);
				os << setfill(' ') << setw(6) << i; 
				TagFormat::writeEndTag (os, "lastframe");
			} // if
		} // for

		TagFormat::writeEndTag (os, "FADE_IN", 1);
		os << endl;
	} // if

	if (data.numberOfFadeOut > 0) {
		TagFormat::writeStartTag (os, "FADE_OUT", 1);
		os << endl;

		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_FADE_OUT) {
				int32 left=i;
				while ((i<data.cuts.getSize()[DIM_X]-1) && (data.cuts(i+1) == SHOT_CHANGE_FADE_OUT)) i++;

				TagFormat::writeStartTag (os, "firstframe", 2);
				os << setfill(' ') << setw(6) << left; 
				TagFormat::writeEndTag (os, "firstframe", -1);

				TagFormat::writeStartTag (os, "lastframe", 2);
				os << setfill(' ') << setw(6) << i; 
				TagFormat::writeEndTag (os, "lastframe");
			} // if
		} // for

		TagFormat::writeEndTag (os, "FADE_OUT", 1);
		os << endl;
	} // if

	// write dissolves
	if (data.numberOfDissolves > 0) {
		TagFormat::writeStartTag (os, "DISSOLVE", 1);
		os << endl;

		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if ((data.cuts(i) == SHOT_CHANGE_DISSOLVE) || 
				(data.cuts(i) == SHOT_CHANGE_DISSOLVE_FIRST) || 
				(data.cuts(i) == SHOT_CHANGE_DISSOLVE_LAST)) {

				int32 left=i;
				while ((i<data.cuts.getSize()[DIM_X]-1) && 
					   ((data.cuts(i+1) == SHOT_CHANGE_DISSOLVE) || 
					    (data.cuts(i) == SHOT_CHANGE_DISSOLVE_FIRST) || 
				     	(data.cuts(i) == SHOT_CHANGE_DISSOLVE_LAST))) i++;
 
				TagFormat::writeStartTag (os, "firstframe", 2);
				os << setfill(' ') << setw(6) << left; 
				TagFormat::writeEndTag (os, "firstframe", -1);

				TagFormat::writeStartTag (os, "lastframe", 2);
				os << setfill(' ') << setw(6) << i; 
				TagFormat::writeEndTag (os, "lastframe");				
			} // if
		} // for

		TagFormat::writeEndTag (os, "DISSOLVE", 1);
		os << endl;
	} // if

	// write wipes
	if (data.numberOfWipes> 0) {
		TagFormat::writeStartTag (os, "WIPE", 1);
		os << endl;

		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if ((data.cuts(i) == SHOT_CHANGE_WIPE) || 
				(data.cuts(i) == SHOT_CHANGE_WIPE_FIRST) || 
				(data.cuts(i) == SHOT_CHANGE_WIPE_LAST)) {

				int32 left=i;
				while ((i<data.cuts.getSize()[DIM_X]-1) && 
					   ((data.cuts(i+1) == SHOT_CHANGE_WIPE) || 
					    (data.cuts(i) == SHOT_CHANGE_WIPE_FIRST) || 
				     	(data.cuts(i) == SHOT_CHANGE_WIPE_LAST))) i++;
 
				TagFormat::writeStartTag (os, "firstframe", 2);
				os << setfill(' ') << setw(6) << left; 
				TagFormat::writeEndTag (os, "firstframe", -1);

				TagFormat::writeStartTag (os, "lastframe", 2);
				os << setfill(' ') << setw(6) << i; 
				TagFormat::writeEndTag (os, "lastframe");				
			} // if
		} // for

		TagFormat::writeEndTag (os, "WIPE", 1);
		os << endl;
	} // if


	// write other scene changes
	if (data.numberOfUnknownSceneChanges > 0) {
		TagFormat::writeStartTag (os, "OTHER_SCENE_CHANGE", 1);
		os << endl;

		for (int32 i=0; i<data.cuts.getSize()[DIM_X]; i++) {
			if (data.cuts(i) == SHOT_CHANGE_OTHER) {

				TagFormat::writeStartTag (os, "frame", 2);
				os << setfill(' ') << setw(6) << i; 
				TagFormat::writeEndTag (os, "frame");
			} // if
		} // for

		TagFormat::writeEndTag (os, "OTHER_SCENE_CHANGE", 1);
		os << endl;
	} // if




    TagFormat::writeEndTag (os, "SCENE_CHANGE");
	os << endl;
	
    return os;
} // operator<<


ITagStream& operator>> (ITagStream& is, CutDetectionData& data)
{
	
    char c;
    bool failFlag = true;
	
	if (is >> c) {
		failFlag = false;
		is.putback(c);

		TagFormat::readStartTag (is, "SCENE_CHANGE");
		
		TagFormat::readNodeValue(is, "version", data.version);
		TagFormat::readNodeValue(is, "filename", data.filename);
		TagFormat::readNodeValue(is, "numberHardCuts", data.numberOfHardCuts);
		TagFormat::readNodeValue(is, "numberFadeIn", data.numberOfFadeIn);
		TagFormat::readNodeValue(is, "numberFadeOut", data.numberOfFadeOut);
		TagFormat::readNodeValue(is, "numberDissolves", data.numberOfDissolves);
		TagFormat::readNodeValue(is, "numberWipes", data.numberOfWipes);
		TagFormat::readNodeValue(is, "numberOther", data.numberOfUnknownSceneChanges);

		TagFormat::readNodeValue(is, "threshold", data.threshold);
		TagFormat::readNodeValue(is, "thresholdHardcut", data.thHardCut);
		TagFormat::readNodeValue(is, "numberOfFrames", data.numberOfFrames);
		if (is.rdstate() & ios::failbit) {
			throw MocaException ("CutDetection: Illegal index file format!");
		} // if

		// initialize size of cuts
		data.cuts.resize(SignalSize(data.numberOfFrames));
		for (int32 i=0; i<data.numberOfFrames; i++) {
			data.cuts(i) = SHOT_CHANGE_UNKNOWN;
		} // for

		// read cuts
		if (is >> c) {
			is.putback(c);

			// read hard cuts
			if (data.numberOfHardCuts>0) {
				TagFormat::readStartTag(is, "HARD_CUT");

				for (int32 i=0; i<data.numberOfHardCuts; i++) {
					int32 p;
					TagFormat::readNodeValue(is, "frame", p);
					data.cuts(p) = SHOT_CHANGE_HARD_CUT;
				} // for

				TagFormat::readEndTag(is, "HARD_CUT");
			} // if
			if (is.rdstate() & ios::failbit) {
				cerr << "CutDetection: Illegal index file format. Wrong number of hard cuts!" << endl << endl;
				exit(1);
			} // if

/*if (data.cuts(7297) == SHOT_CHANGE_HARD_CUT) {
	cout << "+" << endl;
} else {
	cout << "-" << endl;
}*/

			// read fades
			if (data.numberOfFadeIn>0) {
				TagFormat::readStartTag(is, "FADE_IN");

				for (int32 i=0; i<data.numberOfFadeIn; i++) {
					int32 p1, p2;
					TagFormat::readNodeValue(is, "firstframe", p1);
					TagFormat::readNodeValue(is, "lastframe",  p2);
					for (int32 k=p1; k<=p2; k++) {
						data.cuts(k) = SHOT_CHANGE_FADE_IN;
					} // for
				} // for

				TagFormat::readEndTag(is, "FADE_IN");
			} // if
			if (is.rdstate() & ios::failbit) {
				cerr << "CutDetection: Illegal index file format. Wrong number of fade ins!" << endl << endl;
				exit(1);
			} // if


			if (data.numberOfFadeOut>0) {
				TagFormat::readStartTag(is, "FADE_OUT");

				for (int32 i=0; i<data.numberOfFadeOut; i++) {
					int32 p1, p2;
					TagFormat::readNodeValue(is, "firstframe", p1);
					TagFormat::readNodeValue(is, "lastframe",  p2);
					for (int32 k=p1; k<=p2; k++) {
						data.cuts(k) = SHOT_CHANGE_FADE_OUT;
					} // for
				} // for

				TagFormat::readEndTag(is, "FADE_OUT");
			} // if
			if (is.rdstate() & ios::failbit) {
				cerr << "CutDetection: Illegal index file format. Wrong number of fade outs!" << endl << endl;
				exit(1);
			} // if

			// read dissolves
			if (data.numberOfDissolves>0) {
				TagFormat::readStartTag(is, "DISSOLVE");

				for (int32 i=0; i<data.numberOfDissolves; i++) {
					int32 p1, p2;
					TagFormat::readNodeValue(is, "firstframe", p1);
					TagFormat::readNodeValue(is, "lastframe",  p2);
					for (int32 k=p1; k<=p2; k++) {
						data.cuts(k) = SHOT_CHANGE_DISSOLVE;
					} // for
				} // for

				TagFormat::readEndTag(is, "DISSOLVE");
			} // if
			if (is.rdstate() & ios::failbit) {
				cerr << "CutDetection: Illegal index file format. Wrong number of dissolves!" << endl << endl;
				exit(1);
			} // if


			// read wipes
			if (data.numberOfWipes>0) {
				TagFormat::readStartTag(is, "WIPE");

				for (int32 i=0; i<data.numberOfWipes; i++) {
					int32 p1, p2;
					TagFormat::readNodeValue(is, "firstframe", p1);
					TagFormat::readNodeValue(is, "lastframe",  p2);
					for (int32 k=p1; k<=p2; k++) {
						data.cuts(k) = SHOT_CHANGE_WIPE;
					} // for
				} // for

				TagFormat::readEndTag(is, "WIPE");
			} // if
			if (is.rdstate() & ios::failbit) {
				cerr << "CutDetection: Illegal index file format. Wrong number of wipes!" << endl << endl;
				exit(1);
			} // if

			// read other cuts
			if (data.numberOfUnknownSceneChanges>0) {
				TagFormat::readStartTag(is, "OTHER_SCENE_CHANGE");

				for (int32 i=0; i<data.numberOfUnknownSceneChanges; i++) {
					int32 p;
					TagFormat::readNodeValue(is, "frame", p);
					data.cuts(p) = SHOT_CHANGE_OTHER;
				} // for

				TagFormat::readEndTag(is, "OTHER_SCENE_CHANGE");
			} // if
			if (is.rdstate() & ios::failbit) {
				cerr << "CutDetection: Illegal index file format. Wrong number of other shots!" << endl << endl;
				exit(1);
			} // if

		} // if
		
		TagFormat::readEndTag (is, "SCENE_CHANGE");
	} // if
	
	if (is.rdstate() & ios::failbit) {
		failFlag = true;
	} // if
    if (failFlag) {
		throw MocaException ("CutDetection: Illegal index file format!");
	} else {
        is.clear();
    } // else

/*if (data.cuts(7297) == SHOT_CHANGE_HARD_CUT) {
	cout << "+" << endl;
} else {
	cout << "-" << endl;
}*/

    return is;
} // operator>> 





CutDetection::CutDetection(const string &pathFilename) 
{
	this->pathFilename = pathFilename;
	storeImages= false;
	maxContrast= false;
	detectFades= true;
	detectDissolves= true;
	for (int32 i=0; i<32; i++) {
		histInitialized[i]=false;
	} // for
} 

CutDetection::CutDetection(const string &pathFilename, bool detectFades, bool detectDissolves, bool storeImages, bool maxContrast) 
{
	this->pathFilename = pathFilename;
	this->detectFades= detectFades;
	this->detectDissolves= detectDissolves;
	this->storeImages= storeImages;
	this->maxContrast= maxContrast;
	for (int32 i=0; i<32; i++) {
		histInitialized[i]=false;
	} // for
} 


void 
CutDetection::process(CutDetectionData& data) 
{
	// read data
	ReadMocaData<CutDetectionData> reader (pathFilename, CUT_DETECTION_FILENAME);
	bool dataValid = reader.process (CUT_DETECTION_VERSION, data); 
	
	if (!dataValid) {
		
		// create new index
		cerr << "\rCutDetection: Creating index file ...          " << endl;
		readValues(data);
		
		// save index file
		WriteMocaData<CutDetectionData> writer (pathFilename, CUT_DETECTION_FILENAME);
		writer.process(data);
		cerr << "\rCutDetection: Creating index file ... done          " << endl;
	} // if

	if (storeImages) {
		if (maxContrast) {
			saveContrastImages (data);
		} else {
			saveJpegImages (data);
		} // else
	} // if
} // process


void 
CutDetection::getShotFrameNumbers (CutDetectionData& data, Signal<int32>& shotFrameNumber) 
{
	// make a copy of data
	CutDetectionData tmpData = data;
	for (int32 i=0; i<data.numberOfFrames; i++) {
		if (tmpData.cuts(i) == SHOT_CHANGE_NO_CUT) {
			tmpData.cuts(i) = SHOT_CHANGE_UNKNOWN;
		} // if
	} // for


	SignalSize size(data.numberOfFrames,3);
	shotFrameNumber.resize(size);
	for (int32 i=0; i<size[DIM_X]; i++) {
		shotFrameNumber(i,0) = 0;
		shotFrameNumber(i,1) = 0;
		shotFrameNumber(i,2) = 0;
	} // for

	int32 firstFrame=0;
	int32 lastFrame=0;
	for (int32 i=0; i<size[DIM_X]; i++) {
		while ((i<size[DIM_X]) && 
			   (tmpData.cuts(i) != SHOT_CHANGE_UNKNOWN)) {
			i++;
		} // while
		firstFrame=i;
		if ((firstFrame>0) && (tmpData.cuts(firstFrame-1) == SHOT_CHANGE_HARD_CUT)) firstFrame--;
		while ((i<tmpData.numberOfFrames) && (tmpData.cuts(i) == SHOT_CHANGE_UNKNOWN)) {
			i++;
		} // while
		lastFrame=i-1;
		if (firstFrame < 0) {
			firstFrame=0;
			cerr << "----------------------------------------------------" << endl;
			cerr << "set firstframe=0 " << endl;
			cerr << "----------------------------------------------------" << endl;
		} // if
		if (lastFrame > size[DIM_X]-1) {
			lastFrame=size[DIM_X]-1;
			cerr << "----------------------------------------------------" << endl;
			cerr << "set lastframe " << endl;
			cerr << "----------------------------------------------------" << endl;
		} // if
		if (firstFrame<lastFrame) {
			int32 referenceFrame = (lastFrame+firstFrame)/2;
			shotFrameNumber(referenceFrame,0) = 1;
			shotFrameNumber(referenceFrame,1) = firstFrame;
			shotFrameNumber(referenceFrame,2) = lastFrame;
		} // if
	} // for

	return;
} // 


void 
CutDetection::getShotFrameNumbersContrast (CutDetectionData& data, Signal<int32>& shotFrameNumber) 
{
	// make a copy of data
	CutDetectionData tmpData = data;
	for (int32 i=0; i<data.numberOfFrames; i++) {
		if (tmpData.cuts(i) == SHOT_CHANGE_NO_CUT) {
			tmpData.cuts(i) = SHOT_CHANGE_UNKNOWN;
		} // if
	} // for


	SignalSize size(data.numberOfFrames,3);
	shotFrameNumber.resize(size);
	for (int32 i=0; i<size[DIM_X]; i++) {
		shotFrameNumber(i,0) = 0;
		shotFrameNumber(i,1) = 0;
		shotFrameNumber(i,2) = 0;
	} // for

	// get contrast
	VideoStatisticData vsdata;
	VideoStatistic vsf (pathFilename);
	vsf.process (vsdata);
#if 0
	EdgeDetectionFilter<uint8> edf;
	VideoDecoder reader (pathFilename);
#endif

	int32 firstFrame=0;
	int32 lastFrame=0;
	for (int32 i=0; i<size[DIM_X]; i++) {
		while ((i<size[DIM_X]) && 
			   (tmpData.cuts(i) != SHOT_CHANGE_UNKNOWN)) {
			i++;
		} // while
		firstFrame=i;
		if ((firstFrame>0) && (tmpData.cuts(firstFrame-1) == SHOT_CHANGE_HARD_CUT)) firstFrame--;
		while ((i<tmpData.numberOfFrames) && (tmpData.cuts(i) == SHOT_CHANGE_UNKNOWN)) {
			i++;
		} // while
		lastFrame=i-1;
		if (firstFrame < 0) {
			firstFrame=0;
			cerr << "----------------------------------------------------" << endl;
			cerr << "set firstframe=0 " << endl;
			cerr << "----------------------------------------------------" << endl;
		} // if
		if (lastFrame > size[DIM_X]-1) {
			lastFrame=size[DIM_X]-1;
			cerr << "----------------------------------------------------" << endl;
			cerr << "set lastframe " << endl;
			cerr << "----------------------------------------------------" << endl;
		} // if
		if (firstFrame<lastFrame) {
			int32 referenceFrame = (lastFrame+firstFrame)/2;
			shotFrameNumber(referenceFrame,1) = firstFrame;
			shotFrameNumber(referenceFrame,2) = lastFrame;


#if 0
			// get frame with higest contrast
			int32 maxIdx = referenceFrame;
			int32 maxV = vsdata.imgData[maxIdx].contrastScale;
			for (int32 i=firstFrame+1; i<lastFrame-1; i++) {
				int32 tmpV = vsdata.imgData[i].contrastScale;
				if (maxV<tmpV) {
					maxIdx = i;
					maxV = vsdata.imgData[maxIdx].contrastScale;
				} // for
			} // for
#endif

			// get frame with higest summarized edge value
			int32 maxIdx = referenceFrame;
			int32 maxV = vsdata.imgData[maxIdx].avgEdgeStrength;
			for (int32 i=firstFrame+1; i<lastFrame-1; i++) {
				int32 tmpV = vsdata.imgData[i].avgEdgeStrength;
				if (maxV<tmpV) {
					maxIdx = i;
					maxV = vsdata.imgData[maxIdx].avgEdgeStrength;
				} // for
			} // for

			referenceFrame = maxIdx;
			shotFrameNumber(referenceFrame,0) = 1;
		} // if
	} // for

	return;
} // 


void 
CutDetection::saveJpegImages (CutDetectionData& data) 
{
	// get path for images
	CreateMocaPath mocaPath (pathFilename);
	string destinationPath = mocaPath.destinationPath;


	// mark frames, which will be stored
	// newImage: false=ignore file, true:store file
	Signal<int32> tmp;
	SignalSize newImageSize (data.numberOfFrames);
	Signal<bool> newImage (newImageSize);

	getShotFrameNumbers (data, tmp);
	for (int32 i=0; i<data.numberOfFrames; i++) {
		newImage(i)=false;
		if (tmp(i,0) != 0) {
			newImage(i)=true;
		} // if
	} // for


	/*
	newImage.resize(data.numberOfFrames);
	for (int32 i=0; i<data.numberOfFrames; i++) 
		newImage(i)=false;
	int32 firstFrame=0;
	int32 frame=0;
	for (int32 i=0; i<data.numberOfFrames; i++) {
		while ((i<data.numberOfFrames) && 
			   ((data.cuts(i) == SHOT_CHANGE_UNKNOWN) || (data.cuts(i) == SHOT_CHANGE_NO_CUT)))
			i++;
		frame = (firstFrame+i) /2;
		while ((i<data.numberOfFrames) && 
			(data.cuts(i) != SHOT_CHANGE_UNKNOWN) && (data.cuts(i) != SHOT_CHANGE_NO_CUT))
			i++;
		firstFrame = i;
		
		newImage(frame)=true;
	} // for
	*/


	// mark already existing frames
	Signal<bool> existingImage;
	existingImage.resize(data.numberOfFrames);
	for (int32 i=0; i<data.numberOfFrames; i++) 
		existingImage(i)=false;

	// read all JPEG files in directory
	bool error=false;
	vector<FileInfo> jpgFiles;
	FileList files(destinationPath, false);    
	files.process("img*.jpg", jpgFiles);
    vector<FileInfo>::iterator it;
	for (it = jpgFiles.begin(); it != jpgFiles.end(); it++) {
		string name = (*it).shortName;

		// tolower
		for (size_t i=0; i< name.length(); i++) {
			name[i] = tolower(name[i]);  
		} // for

		name=name.substr(3);
		name=name.substr(0,name.size()-4);
		int32 num=atoi(name.c_str());
		if ((num<0) || (num>data.numberOfFrames)) {
			error=true;
		} else {
			existingImage(num)=true;
		} // else
		//img00000175.jpg
	} // for

	if (error) {

		// delete all old "img*.jpg" files
		string tmpFile = destinationPath + "img*.jpg";
		FileHandling::deleteFile (tmpFile);

		for (int32 i=0; i<data.numberOfFrames; i++) {
			existingImage(i)=false;
		} // for
	} // if



	// compare newImage and existingImage
	bool updateRequired=false;
	for (int32 i=0; i<data.numberOfFrames; i++) {
		if ((existingImage(i)) && (newImage(i))) {
			existingImage(i)=false;
			newImage(i)=false;
		} // if
		if (newImage(i)) updateRequired=true;
	} // for


	// save images
	if (updateRequired) {
		// find last frame
		int32 lastFrame=-1;
		for (int32 i=data.numberOfFrames-1; i>0; i--) {
			if (newImage(i)) {
				lastFrame=i+1;
				break;
			} // if
		} // for
		
		// open mpeg file
		int cacheSize=3;
		VideoCodec codec = CODEC_RGB;
		VideoDecoder reader (pathFilename, cacheSize, lastFrame, codec);
		Image<uint8> img;
		
		for (int32 i=0; i<data.numberOfFrames; i++) {
			if (newImage(i)) {
				
				// read image
				img=reader.getImage(i);
				
				// save images
                {
                    stringstream oss;
                    oss << destinationPath << "img" << setw(8) << setfill('0') << (i) << ".jpg";
                    cout << "save image: " << oss.str() << endl;
                    OImageStream<uint8> ois(oss.str());
                    ois.write (img);
                }
                
			} // if
		} // for
	} // if
	

	return;
} // saveJpegImages 

void 
CutDetection::saveContrastImages (CutDetectionData& data) 
{
	// get path for images
	CreateMocaPath mocaPath (pathFilename);
	string destinationPath = mocaPath.destinationPath;


	// mark frames, which will be stored
	// newImage: false=ignore file, true:store file
	Signal<int32> tmp;
	SignalSize newImageSize (data.numberOfFrames);
	Signal<bool> newImage (newImageSize);

	getShotFrameNumbersContrast (data, tmp);
	for (int32 i=0; i<data.numberOfFrames; i++) {
		newImage(i)=false;
		if (tmp(i,0) != 0) {
			newImage(i)=true;
		} // if
	} // for


	// mark already existing frames
	Signal<bool> existingImage;
	existingImage.resize(data.numberOfFrames);
	for (int32 i=0; i<data.numberOfFrames; i++) 
		existingImage(i)=false;

	// read all PNG files in directory
	bool error=false;
	vector<FileInfo> pngFiles;
	FileList files(destinationPath, false);    
	files.process("img*.png", pngFiles);
    vector<FileInfo>::iterator it;
	for (it = pngFiles.begin(); it != pngFiles.end(); it++) {
		string name = (*it).shortName;

		// tolower
		for (size_t i=0; i< name.length(); i++) {
			name[i] = tolower(name[i]);  
		} // for

		name=name.substr(3);
		name=name.substr(0,name.size()-4);
		int32 num=atoi(name.c_str());
		if ((num<0) || (num>data.numberOfFrames)) {
			error=true;
		} else {
			existingImage(num)=true;
		} // else
		//img00000175.png
	} // for

	if (error) {

		// delete all old "img*.png" files
		string tmpFile = destinationPath + "img*.png";
		FileHandling::deleteFile (tmpFile);

		for (int32 i=0; i<data.numberOfFrames; i++) {
			existingImage(i)=false;
		} // for
	} // if



	// compare newImage and existingImage
	bool updateRequired=false;
	for (int32 i=0; i<data.numberOfFrames; i++) {
		if ((existingImage(i)) && (newImage(i))) {
			existingImage(i)=false;
			newImage(i)=false;
		} // if
		if (newImage(i)) updateRequired=true;
	} // for


	// save images
	if (updateRequired) {
		// find last frame
		int32 lastFrame=-1;
		for (int32 i=data.numberOfFrames-1; i>0; i--) {
			if (newImage(i)) {
				lastFrame=i+1;
				break;
			} // if
		} // for
		
		// open mpeg file
		int cacheSize=3;
		VideoCodec codec = CODEC_RGB;
		VideoDecoder reader (pathFilename, cacheSize, lastFrame, codec);
		Image<uint8> img;
		
		for (int32 i=0; i<data.numberOfFrames; i++) {
			if (newImage(i)) {
				
				// read image
				img=reader.getImage(i);
				
				// save images
                {
                    stringstream oss;
                    oss << destinationPath << "img" << setw(8) << setfill('0') << (i) << ".png";
                    cout << "save image: " << oss.str() << endl;
                    OImageStream<uint8> ois(oss.str());
                    ois.write (img);
                }
                
			} // if
		} // for
	} // if
	

	return;
} // saveContrastImages 


void 
CutDetection::initHistogram (int32 skip) 
{
	int32 bits = 4;
	bool grayscale = true;	
	bool optimize = true;	

	if (!histInitialized[skip]) {
		HistogramDifferenceFeature hdf (pathFilename, bits, skip, grayscale, optimize);
		hdf.process(hist[skip]);
		histInitialized[skip]=true;
	} // if

	return;
} // initHistogram


void 
CutDetection::findHardCuts (CutDetectionData& data) 
{
	// read histogram values
	initHistogram (0);

	// initialize variables
	data.numberOfHardCuts = 0;
	data.threshold = 2.0 * hist[0].avgHistValue;
	
	// A hard cuts is a peak "j" of hist[0].values[j] with the length of one.
	// The peak "j" should be significant larger than the area around the peak:
	//    hist[0].values[j] > max (hist[0].values[j-5]...hist[0].values[j+5]) + threshold	
	for (uint32 j=5; j< hist[0].frames-5; j++) {

		// current value is large enough
		if (hist[0].values[j] > data.threshold) {
			
			// get maximum value in neigbourhood
			ieee64 maxV = hist[0].values[j-5];
			for (int32 k=1; k<6; k++) {
				if (maxV <hist[0].values[j-k]) {
					maxV = hist[0].values[j-k];	
				} // if
				if (maxV <hist[0].values[j+k]) {
					maxV = hist[0].values[j+k];	
				} // if
			} // if
			
			// add theshold and compare current value
			maxV += data.threshold;
			//ieee64 tttemp = hist[0].values[j];
			if (hist[0].values[j] > maxV) {
				data.numberOfHardCuts++;
				
				// mark following frame as hard cut (frame number of next frame)
				for (uint32 u=j-4; u<j+7; u++) {
					data.cuts(u) = SHOT_CHANGE_NO_CUT;
				} // for
				data.cuts(j+1) = SHOT_CHANGE_HARD_CUT;
			} // if
		} // if
	} // for


	
	// get average histogram values at hard cut position
	data.thHardCut=0.0;
	ieee64 nHardCut =0.0;
	for (int32 j=0; j<data.cuts.getSize()[DIM_X]; j++) {
		if (data.cuts(j) == SHOT_CHANGE_HARD_CUT) {
			data.thHardCut+=hist[0].values[j-1];
			nHardCut++;
		} // if
	} // for
	data.numberOfHardCuts = (int32) nHardCut;
	if (nHardCut<=0.0) nHardCut=1.0;
	data.thHardCut/=nHardCut;
	if (data.thHardCut < 2.0*data.threshold) {
		data.thHardCut=2.0*data.threshold;
	} // if

	return;
} // detectHardCuts



void
CutDetection::removeFalseHardCuts (CutDetectionData& data) 
{
	// compare histograms of frames i and i+2
	//     if both frames are very similar, the relevant frame has much noise, 
	//     image errors, a large and fast moving object in the front, ...

	// read histogram values
	initHistogram (1);

	int32 removedCuts = 0;
	for (int32 frame=5; frame< data.cuts.getSize()[DIM_X]-5; frame++) {
		if (data.cuts(frame) == SHOT_CHANGE_HARD_CUT) {

			// get maximum value in neigbourhood
			ieee64 maxV = hist[1].values[frame];
			for (int32 k=0; k<4; k++) {
				if (maxV < hist[1].values[frame+k]) {
					maxV = hist[1].values[frame+k];	
				} // if
				if (maxV < hist[1].values[frame-3-k]) {
					maxV = hist[1].values[frame-3-k];	
				} // if
			} // if
		
			// TODO: modified (TEST!!!)
			// add theshold 
			//maxV += data.threshold;
			maxV += data.threshold / 4.0;

			// get minimum histogram value of double peaks
			ieee64 peakValue = hist[1].values[frame-1];
			if (peakValue > hist[1].values[frame-2]) {
				peakValue = hist[1].values[frame-2];
			} // if

			// compare peakValue with maxV (threshold)
			if (peakValue < maxV) {
				// remove hard cut
				removedCuts++;
				
				for (int32 u=frame-5; u<frame+6; u++) {
					if ((data.cuts(u) == SHOT_CHANGE_HARD_CUT) || (data.cuts(u) == SHOT_CHANGE_NO_CUT)) {
						data.cuts(u) = SHOT_CHANGE_UNKNOWN;
					} // if
				} // for
			} // if
		} // if
	} // for

	//data.numberOfHardCuts -= removedCuts;

	// update hard cuts
	if (removedCuts != 0) {

		// get average histogram values at hard cut position
		data.thHardCut=0.0;
		ieee64 nHardCut =0.0;
		for (int32 j=0; j<data.cuts.getSize()[DIM_X]; j++) {
			if (data.cuts(j) == SHOT_CHANGE_HARD_CUT) {
				data.thHardCut+=hist[0].values[j-1];
				nHardCut++;
			} // if
		} // for
		data.numberOfHardCuts = (int32) nHardCut;
		if (nHardCut<=0.0) nHardCut=1.0;
		data.thHardCut/=nHardCut;
		if (data.thHardCut < 2.0*data.threshold) {
			data.thHardCut=2.0*data.threshold;
		} // if
	} // if
	
	
	return;
} // removeFalseHardCuts





void 
CutDetection::findFades (CutDetectionData& data) 
{
	/*
	// read video statistics
	VideoStatisticData item;
	VideoStatistic vs (filename);
	vs.process(item);

	// calculate (summarize) values 
	Signal<int32> sig (SignalSize(item.frames));
	//ieee64 maxValue = -1;
	for (int32 i=0; i<item.frames; i++) {
		sig(i) = (int32) (item.imgData(i).sigma[0] + 4.0*item.imgData(i).sigma[1] + 4.0*item.imgData(i).sigma[2]);
	} // for
	*/

	// define threshold 
	const int32 thHigh       = 25; // compare sigma with threshold
	const int32 thLow        = 15; // compare sigma with threshold
	const int32 thLengthHigh =  2; // compare length with threshold
	const int32 thHeight     =  1; // compare sigma differences with threshold
	const int32 thFadeLength =  3; // compare length with threshold
	const int32 thDiffCenterBorder =  10;  // compare sigma differences with threshold

	// find fades 
	int32 left   = 0;
	int32 right  = 0;
	int32 centerLeft = 0;
	int32 centerRight = 0;
	int32 i=0;
	while (i<sigmaSize[DIM_X]) {
		
		// locate first monochrome frame
		while ((i<sigmaSize[DIM_X]) && (sigma(i) >= thHigh)) {
			i++;
		} // while
		left = i;
		
		// locate last monochrome frame
		while ((i<sigmaSize[DIM_X]) && (sigma(i) < thHigh)) {
			i++;
		} // while
		right = i-1;
		//int32 centerTmp = (left+right)/2;
		
		// locate exact position of left and right
		while ((left>0) && (sigma(left-1) >= sigma(left)+thHeight)) {
			left--;
		} // while
		while ((right<sigmaSize[DIM_X]-1) && (sigma(right+1) >= sigma(right) + thHeight)) {
			right++;
		} // while

		// locate end of fade (leftCenter, rightCenter)
		centerLeft = left;
		while ((centerLeft<sigmaSize[DIM_X]-1) && 
			   (sigma(centerLeft+1) <= sigma(centerLeft)) && 
			   (!cutExist (data, centerLeft+1, centerLeft+1))) {
			centerLeft++;
		} // while
		centerRight = right;
		while ((centerRight>0) && 
			   (sigma(centerRight-1) <= sigma(centerRight)) && 
			   (!cutExist (data, centerRight-1, centerRight-1))) {
			centerRight--;
		} // while

		// fade at the begin or end of the video?
		if (left <= 0) {
			left   = 0;
			centerLeft = 0;
		} // if
		if (right >= sigmaSize[DIM_X]-1) {
			right   = sigmaSize[DIM_X]-1;
			centerRight = sigmaSize[DIM_X]-1;
		} // if


		// fade out? (between left and centerLeft)
		bool isFadeOut = false;
		if (centerLeft - left > thFadeLength) {
			if (!cutExist (data, left-1, centerLeft)) {
				int32 n=0;
				for (int32 k=left; k<=centerLeft; k++) {
					if (sigma(k) < thHigh) {
						n++;
					} // if
					if (sigma(k) < thLow) {
						isFadeOut = true;
					} // if
				} // for
				if (n > thLengthHigh) { 
					isFadeOut = true;
				} // if
				if (isFadeOut) {
					// compare left and centerLeft values
					if (sigma(left) - sigma(centerLeft) <= thDiffCenterBorder) {
						isFadeOut=false;
					} // if
				} // if
			} // if
		} // if

		// fade in ? (between centerRight and right)
		bool isFadeIn = false;
		if (right - centerRight > thFadeLength) {
			if (!cutExist (data, centerRight, right+1)) {
				int32 n=0;
				for (int32 k=centerRight; k<=right; k++) {
					if (sigma(k) < thHigh) {
						n++;
					} // if
					if (sigma(k) < thLow) {
						isFadeIn = true;
					} // if
				} // for
				if (n > thLengthHigh) { 
					isFadeIn = true;
				} // if
				if (isFadeIn) {
					// compare right and centerRight values
					if (sigma(right) - sigma(centerRight) <= thDiffCenterBorder) {
						isFadeIn=false;
					} // if
				} // if
			} // if
		} // if


		// set data
		if (isFadeOut) {
			for (int32 k=left; k<=centerLeft; k++) {
				if (!cutExist(data, k, k)) {
					data.cuts(k) = SHOT_CHANGE_FADE_OUT;
				} // if
			} // for
		} // if
		if (isFadeIn) {
			for (int32 k=centerRight; k<=right; k++) {
				if (!cutExist(data, k, k)) {
					data.cuts(k) = SHOT_CHANGE_FADE_IN;
				} // if
			} // for
		} // if
	} // while

	SignalSize size=data.cuts.getSize();

/*
	// detect exact border of fades
	initHistogram (0);
	ieee64 th = hist[0].avgHistValue;

	SignalSize size=data.cuts.getSize();
	for (int32 i=size[DIM_X]-1; i>=0; i--) {
		if (data.cuts(i) == SHOT_CHANGE_FADE_OUT) {

			int32 right = i;
			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_OUT)) {
				i--;
			} // for

			// get exact right border of fade
			int32 frame = right;
			while ((frame<size[DIM_X]-1) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_OUT)) &&
				   (hist[0].values[frame]>th) ){ 
				frame++;
			} // while

			// fade out has monochrome frames at the end of the fade
			frame++;
			ieee64 thMod = th;
			while ((frame < size[DIM_X]-1) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_OUT)) &&
				   (hist[0].values[frame] < thMod) ){
				thMod*=0.9;
				frame++;
			} // while
			frame--;

			right=frame-1;

			// get exact left border of fade
			frame = i;
			while ((frame>0) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_OUT)) &&
				   (hist[0].values[frame]>3.0*th) ){ 
				frame--;
			} // while
			while ((frame>0) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_OUT)) &&
				   (hist[0].values[frame]>th) && 
				   (hist[0].values[frame]>hist[0].values[frame-1])){
				frame--;
			} // while
			// set fade out
			for (int32 j=frame; j<=right; j++) {
				data.cuts(j) = SHOT_CHANGE_FADE_OUT;
			} // for

			i=frame;
			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_OUT)) {
				i--;
			} // for
		} // if

		if (data.cuts(i) == SHOT_CHANGE_FADE_IN) {

			int32 right = i;
			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_IN)) {
				i--;
			} // for

			// get exact right border of soft cut
			int32 frame = right;
			while ((frame<size[DIM_X]-1) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_IN)) &&
				   (hist[0].values[frame]>3.0*th) ){ 
				frame++;
			} // while
			while ((frame < size[DIM_X]-1) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_IN)) &&
				   (hist[0].values[frame] > th) && 
				   (hist[0].values[frame] > hist[0].values[frame+1])){
				frame++;
			} // while
			right=frame;

			// get exact left border of soft cut
			frame = i;
			while ((frame>0) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_IN)) &&
				   (hist[0].values[frame]>th) ){ 
				frame--;
			} // while
			for (int32 j=frame+1; j<=i; j++) {
				data.cuts(j) = SHOT_CHANGE_FADE_IN;
			} // for
			
			// fade in has monochrome frames at the beginning of the fade
			frame--;
			ieee64 thMod = th;
			while ((frame>0) && 
				   ((data.cuts(frame) == SHOT_CHANGE_UNKNOWN) || (data.cuts(frame) == SHOT_CHANGE_FADE_IN)) &&
				   (hist[0].values[frame] < thMod) ){
				frame--;
				thMod*=0.9;
			} // while
			frame++;
			i=frame;

			// set fade in
			for (int32 j=i; j<=right; j++) {
				data.cuts(j) = SHOT_CHANGE_FADE_IN;
			} // for

			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_IN)) {
				i--;
			} // for
		} // if
	} // for
*/

	// optimize end of video
	int32 lastFadeOut=-1;
	for (int32 i=size[DIM_X]-1; i>size[DIM_X]-60; i--) {
		if (i<=0) break;
		if (data.cuts(i) == SHOT_CHANGE_FADE_OUT) {
			lastFadeOut=i;
			break;
		} // if
		if ((data.cuts(i) != SHOT_CHANGE_UNKNOWN) && (data.cuts(i) != SHOT_CHANGE_NO_CUT)) {
			break;
		} // if
	} // for
	if (lastFadeOut !=-1) {
		for (int32 i=lastFadeOut; i<size[DIM_X]; i++) {
			data.cuts(i) = SHOT_CHANGE_FADE_OUT;
		} // for
	} // if
	
	// count fades
	data.numberOfFadeOut=0;
	data.numberOfFadeIn =0;
	for (int32 i=size[DIM_X]-1; i>0; i--) {
		while ((i>0) && (data.cuts(i) != SHOT_CHANGE_FADE_OUT)) {
			i--;
		} // while
		if (data.cuts(i) == SHOT_CHANGE_FADE_OUT) {
			data.numberOfFadeOut++;
			while ((i>0) && (data.cuts(i) == SHOT_CHANGE_FADE_OUT)) {
				i--;
			} // while
		} // if
	} // for
	for (int32 i=size[DIM_X]-1; i>0; i--) {
		while ((i>0) && (data.cuts(i) != SHOT_CHANGE_FADE_IN)) {
			i--;
		} // while
		if (data.cuts(i) == SHOT_CHANGE_FADE_IN) {
			data.numberOfFadeIn++;
			while ((i>0) && (data.cuts(i) == SHOT_CHANGE_FADE_IN)) {
				i--;
			} // while
		} // if
	} // for

	return;
} // detectFades



ieee64 
CutDetection::getMaxHistValue (HistogramDifferenceItem &hist, int32 p1, int32 p2)
{
	ieee64 v=hist.values[p1];
	for (int32 k=p1+1; k<=p2; k++) {
		if (v <hist.values[k]) {
			v = hist.values[k];	
		} // if
	} // if
	return v;
} // getMaxHistValue 

ieee64 
CutDetection::getMaxHistValue (HistogramDifferenceItem &hist, int32 p1, int32 p2, int32 p3, int32 p4)
{
	ieee64 v1=getMaxHistValue (hist, p1, p2);
	ieee64 v2=getMaxHistValue (hist, p3, p4);
	if (v1>v2) {
		return v1;
	} // if
	return v2;
} // getMaxHistValue 

ieee64 
CutDetection::getMinHistValue (HistogramDifferenceItem &hist, int32 p1, int32 p2)
{
	ieee64 v=hist.values[p1];
	for (int32 k=p1+1; k<=p2; k++) {
		if (v >hist.values[k]) {
			v = hist.values[k];	
		} // if
	} // if
	return v;
} // getMinHistValue 

ieee64 
CutDetection::getMinHistValue (HistogramDifferenceItem &hist, int32 p1, int32 p2, int32 p3, int32 p4)
{
	// TODO check!!!!!!!!!!!!!!  --> changed from getMaxHistValue
	ieee64 v1=getMinHistValue (hist, p1, p2);
	ieee64 v2=getMinHistValue (hist, p3, p4);
	if (v1<v2) {
		return v1;
	} // if
	return v2;
} // getMinHistValue 

ieee64 
CutDetection::getAvgHistValue (HistogramDifferenceItem &hist, int32 p1, int32 p2)
{
	ieee64 v=hist.values[p1];
	for (int32 k=p1+1; k<=p2; k++) {
		v+=hist.values[k];
	} // if
	ieee64 n=p2-p1+1;
	if (n<=0) {
		n=1.0;
		v=0.0;
	} // if
	return v/n;
} // getAvgHistValue 

ieee64 
CutDetection::getAvgHistValue (HistogramDifferenceItem &hist, int32 p1, int32 p2, int32 p3, int32 p4)
{
	ieee64 v=(p2-p1+1)*getAvgHistValue (hist, p1, p2) + (p4-p3+1)*getAvgHistValue (hist, p3, p4);
	ieee64 n=(p4-p3+1)+(p2-p1+1);
	if (n<=0) {
		n=1.0;
		v=0.0;
	} // if
	return v/n;
} // getAvgHistValue 




void 
CutDetection::findShortDissolves (CutDetectionData& data) 
{
	// analyse dissolve areas
	Signal<int32> dissolvePosition;
	getDissolveArea (data, dissolvePosition);

	// read histogram values
	initHistogram (0);
	initHistogram (1);
	initHistogram (2);


	// (1) find dissolves with dissolve-length==2

	// A short dissolveis a cut with 2 frames. The histogram should have a peak of 
	// length 2. 
	// The peak "j" should be significant larger than the area around the peak:
	//    item.values[j] > max (item.values[j-5]...item.values[j+5]) + threshold	


	// idea:
	// histValue(hist[0])   0.1  0.1  0.9  0.9  0.1  0.1
	// histValue(hist[1])   0.1  0.7  1.5  0.9  0.1  0.1
	// position           j-3  j-2  j-1   j   j+1  j+2

	// 5 criteria:
	// set th0=data.thHardCut * 0.5-data.threshold
	// set th1=data.thHardCut * 0.75-data.threshold
	// (a) hist[0].v[j-1..j] > th0
	// (b) no scene change between j-2..j+1
	// (c) v1=min (hist[0].v[j-1], hist[0].v[j])
	//     v2=max (hist[0].v[j-6..j-2],hist[0].v[j+1..j+5])
	//     v1 > v2+data.threshold
	// (d) v3=max (hist[1].v[j-2..j]) 
	//     v3 > th1
	// (e) v4=min (hist[1].v[j-2], hist[1].v[j])
	//     v5=max (hist[1].v[j-7..j-3],hist[1].v[j+1..j+5])
	//     v4 > v5+data.threshold
	

	for (uint32 j=7; j< hist[0].frames-5; j++) {

		bool isDissolve=true;

		// get values
		ieee64 minPeak0=getMinHistValue(hist[0],j-1,j);
		ieee64 avg0=getAvgHistValue(hist[0],j-6,j-2,j+1,j+5);
		ieee64 max0=getMaxHistValue(hist[0],j-6,j-2,j+1,j+5);
		ieee64 avg1=getAvgHistValue(hist[1],j-7,j-3,j+1,j+5);

		// dissolve with length 2: 
		// 2 areas, peaks of hist[0] and hist[1] (P0 and P1) and neighbourhood of these histograms (N0,N1)

		// (a)
		if (minPeak0<avg0+data.threshold) isDissolve=false;

		// (b)
		if (minPeak0<max0) isDissolve=false;

		// (c)
		if (cutExist(data, j-2, j+1)) isDissolve=false;
		
		// (d)
		if (max0>avg0+data.threshold) isDissolve=false;
		
		// (e)
		ieee64 v=getMaxHistValue(hist[1],j-7,j-3,j+1,j+5);
		if (v>avg1+data.threshold) isDissolve=false;
		
		// (f) 
		if (hist[1].values[j-1]<hist[1].values[j]) isDissolve=false;
		if (hist[1].values[j-1]<hist[1].values[j-2]) isDissolve=false;


		if (isDissolve) {

			// (g) analyse dissolve area
			int32 left=-1;
			int32 right=-1;
			if (dissolvePosition(j-1) !=0) left=j-1;
			if (dissolvePosition(j) !=0) left=j;
			if (left==-1) {
				isDissolve=false;
			} else {
				right=left;
				while ((left>0) && ((dissolvePosition(left)%1000)/100 < 1)) {
					left--;
				} // while
				while ((right<dissolvePosition.getSize()[DIM_X]-1) && (dissolvePosition(right) < 10000)) {
					right++;
				} // while
				if (left<(int32)j-6) {
					isDissolve=false;
				} // if
				if (right>(int32)j+4) {
					isDissolve=false;
				} // if
			} // else

			// set value
			if (isDissolve) {				
				data.numberOfDissolves++;
				
				// mark as dissolves
				for (int32 u=left-5; u<=right+5; u++) {
					if (data.cuts(u) == SHOT_CHANGE_UNKNOWN) {
						data.cuts(u) = SHOT_CHANGE_NO_CUT;
					} // if
				} // for
				for (int32 u=left; u<=right; u++) {
					data.cuts(u) = SHOT_CHANGE_DISSOLVE;
				} // for
			} // if 
		} // if
	} // for



	// (2) find dissolves with dissolve-length==3

	// idea: peak values
	// histValue(hist[0])   0.1   0.1   0.1   0.8P  0.7P  0.9P  0.1
	// histValue(hist[1])   0.1   0.1   0.6P  0.7P  1.5P  0.9P  0.1
	// histValue(hist[2])   0.1   0.5P  0.6P  1.2P  1.5P  0.9P  0.1
	// position           j-5   j-4   j-3   j-2   j-1    j    j+1

	// 5 criteria:
	// set th0=data.thHardCut * 0.4-data.threshold
	// set th1=data.thHardCut * 0.6-data.threshold
	// set th2=data.thHardCut * 0.75-data.threshold
	// (a) hist[0].v[j-2..j] > th0
	// (b) no scene change between j-3..j+1
	// (c) v1=min (hist[0].v[j-2]..hist[0].v[j])
	//     v2=max (hist[0].v[j-7..j-3],hist[0].v[j+1..j+5])
	//     v1 > v2+data.threshold
	// (d) v3=max (hist[1].v[j-3..j]) 
	//     v3 > th1
	// (e) v4=min (hist[1].v[j-3]..hist[1].v[j])
	//     v5=max (hist[1].v[j-8..j-4],hist[1].v[j+1..j+5])
	//     v4 > v5+data.threshold
	// (f) v6=max (hist[2].v[j-4..j]) 
	//     v6 > th2
	// (g) v7=min (hist[2].v[j-4]..hist[1].v[j])
	//     v8=max (hist[2].v[j-9..j-5]..hist[2].v[j+1..j+5])
	//     v7 > v8+data.threshold



	// dissolve with length 3: 
	for (uint32 j=7; j< hist[0].frames-5; j++) {

		bool isDissolve=true;

		//if (j>2950) {
			//cout << endl;
		//} // if

		// get values
		ieee64 minPeak0=getMinHistValue(hist[0],j-2,j);
		ieee64 avg0=getAvgHistValue(hist[0],j-7,j-3,j+1,j+5);
		ieee64 max0=getMaxHistValue(hist[0],j-7,j-3,j+1,j+5);
		ieee64 avg1=getAvgHistValue(hist[1],j-8,j-4,j+1,j+5);

		// (a)
		if (minPeak0<avg0+data.threshold) isDissolve=false;

		// (b)
		if (minPeak0<max0) isDissolve=false;

		// (c)
		if (cutExist(data, j-3, j+1)) isDissolve=false;

		// (d)
		if (max0>avg0+data.threshold) isDissolve=false;
		
		// (e)
		ieee64 v=getMaxHistValue(hist[1],j-8,j-4,j+1,j+5);
		if (v>avg1+data.threshold) isDissolve=false;
		
		// (f) 
		v=(hist[1].values[j-1]+hist[1].values[j-2])/2.0;
		if (v<hist[1].values[j]) isDissolve=false;
		if (v<hist[1].values[j-3]) isDissolve=false;

		if (isDissolve) {

			// (g) analyse dissolve area
			int32 left=-1;
			int32 right=-1;
			if (dissolvePosition(j-1) !=0) left=j-1;
			if (dissolvePosition(j) !=0) left=j;
			if (left==-1) {
				isDissolve=false;
			} else {
				right=left;
				while ((left>0) && ((dissolvePosition(left)%1000)/100 < 1)) {
					left--;
				} // while
				while ((right<dissolvePosition.getSize()[DIM_X]-1) && (dissolvePosition(right) < 10000)) {
					right++;
				} // while
				if (left<(int32)j-7) {
					isDissolve=false;
				} // if
				if (right>(int32)j+5) {
					isDissolve=false;
				} // if
			} // else


			// set value
			if (isDissolve) {				
				data.numberOfDissolves++;
				
				// mark as dissolves
				for (int32 u=left-5; u<=right+5; u++) {
					if (data.cuts(u) == SHOT_CHANGE_UNKNOWN) {
						data.cuts(u) = SHOT_CHANGE_NO_CUT;
					} // if
				} // for
				for (int32 u=left; u<=right; u++) {
					data.cuts(u) = SHOT_CHANGE_DISSOLVE;
				} // for
			} // if 
		} // if
	} // for

	return;
} // detectShortDissolves



void 
CutDetection::findDissolves (CutDetectionData& data) 
{
	Signal<int32> dissolveArea;
	getDissolveArea (data, dissolveArea);

	// read data
	for (int32 i=0; i<31; i++) {
		initHistogram (i);
	} // for
	SignalSize size (hist[0].frames);


	// 0: UNKNOWN, 1:dissolve-area, 2:dissolve-position, 3:no-dissolve
	Signal<uint8> dissolve;
	dissolve.resize(size);
	for (int32 i=0; i<size[DIM_X]; i++) {
		dissolve(i) = 0;
	} // for
	//ieee64 th = hist[0].avgHistValue;
	for (int32 i=0; i<size[DIM_X]; i++) {
		if (data.cuts(i) != SHOT_CHANGE_UNKNOWN) {
			dissolve(i) = 3;
		} // if
	} // for


	// remove hard cuts 
	//int32 i=0;
	//const int32 firstT = 0;
	const int32 lastT = 30;
	for (int32 i=size[DIM_X]-1; i>=0; i--) {
		if (data.cuts(i) == SHOT_CHANGE_HARD_CUT) {
			removeSoftCut (hist, i-2, i, lastT);
			//removeHardCut (hist, i, lastT);
		} // if
	} // for


	// remove fades and dissolves
	for (int32 i=size[DIM_X]-1; i>=0; i--) {
		
		if (data.cuts(i) == SHOT_CHANGE_FADE_OUT) {
			int32 right = i+2;
			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_OUT)) {
				i--;
			} // for
			i--;
			if (right>size[DIM_X]-1) right=size[DIM_X]-1;
			if (i<0) i=0;
			removeSoftCut (hist, i, right, lastT);

			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_OUT)) {
				i--;
			} // for
		} // if

		if (data.cuts(i) == SHOT_CHANGE_FADE_IN) {
			int32 right = i+2;
			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_IN)) {
				i--;
			} // for
			i--;
			if (right>size[DIM_X]-1) right=size[DIM_X]-1;
			if (i<0) i=0;
			removeSoftCut (hist, i, right, lastT);

			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_FADE_IN)) {
				i--;
			} // for
		} // if

		if (data.cuts(i) == SHOT_CHANGE_DISSOLVE) {
			int32 right = i+2;
			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_DISSOLVE)) {
				i--;
			} // for
			i--;
			if (right>size[DIM_X]-1) right=size[DIM_X]-1;
			if (i<0) i=0;
			removeSoftCut (hist, i, right, lastT);

			while ((i>0) && (data.cuts(i-1) == SHOT_CHANGE_DISSOLVE)) {
				i--;
			} // for
		} // if
	} // for

	// berechne 1er Spalten
	// calculate average1: vertical summarization
	Signal<ieee64> avg2b;
	avg2b.resize(size);
	for (int32 i=0; i<size[DIM_X]; i++) {
		avg2b(i) = 0.0;		
		ieee64 n=0;
		for (int32 t=5; t<=30; t++) {
			//avg1a(i) += hist[t].values[i-1];
			avg2b(i) += hist[t].values[i];
			//avg1a(i) += hist[t].values[i+1];
			n++;
		} // for
		if (n>0) {
			avg2b(i) /= n;
		} // else
	} // for


	// analusiere avg2b
	for (int32 i=size[DIM_X]-1; i>0; i--) {
		while ((i>0) && (dissolveArea(i) == 0)) i--;

		while ((i>0) && (dissolveArea(i) <10000)) i--;
		int32 right=i;

		while ((i>0) && ((dissolveArea(i)%10000) <1000)) i--;
		//int32 center=i;

		while ((i>0) && ((dissolveArea(i)%1000) <100)) i--;
		int32 left=i;
		i++;
		
		if ((left>0) && (dissolveArea(left) != 0) && (dissolveArea(right) != 0)) {
			
			int32 realMinP=left;
			int32 realMaxP=left;
			int32 maxP=left;
			int32 maxP2=left;
			ieee64 maxV=0.0;

			int32 tmp1=left-2; if (tmp1<1) tmp1=1;
			int32 tmp2=right+2; if (tmp2>size[DIM_X]-1) tmp2=size[DIM_X]-1;
			for (int32 j=tmp1; j<=tmp2; j++) {
				if (avg2b(maxP)<avg2b(j)) maxP=j;
				if (maxV<avg2b(j-1)-avg2b(j)) {
					maxP2=j-1;
					maxV=avg2b(j-1)-avg2b(j);
				} // if
				if (avg2b(realMinP)>avg2b(j)) {
					realMinP=j;
				} // if
				if (avg2b(realMaxP)<avg2b(j)) {
					realMaxP=j;
				} // if
			} // for
			
			if (avg2b(realMaxP) - avg2b(realMinP)> data.threshold) {
				
				ieee64 step = 0.0;
				while ((maxP<size[DIM_X]-1) && (avg2b(maxP) >= avg2b(maxP+1)+step/2.0)) {
					if (step < avg2b(maxP)-avg2b(maxP+1)) {
						step = avg2b(maxP)-avg2b(maxP+1);
					} // if
					if (step>data.threshold) {
						step=data.threshold;
					} // if
					maxP++;
				} // while
				if (maxP<=right+1) {
					
					// is hard cut?
					bool isHardCut=false;
					int32 peakP=left;
					ieee64 peak=0.0;
					ieee64 thHardCut=0.6*data.thHardCut;
					if (right-left < 10) {
						for (int32 k=left; k<=right; k++) {
							ieee64 tmp = abs (hist[0].values[k] - hist[0].values[k-1]);
							if (peak<tmp) {
								peak=tmp;
								peakP=k;
							} // if
						} // for
						if (peak>thHardCut) {
							isHardCut=true;
							i=right+1;
						} // if
					} // if
					
					if (isHardCut) {
						int32 p=left;
						for (int32 k=left; k<=right; k++) {
							if (hist[0].values[p]<hist[0].values[k]) {
								p=k;
							} // if
						} // for
						
						if (((hist[0].values[p+1]<0.5*data.thHardCut) ||
							 (hist[0].values[p+2]<0.5*data.thHardCut) ||
							 (hist[0].values[p+3]<0.5*data.thHardCut)) &&
							((hist[0].values[p-1]<0.5*data.thHardCut) ||
							 (hist[0].values[p-2]<0.5*data.thHardCut) ||
							 (hist[0].values[p-2]<0.5*data.thHardCut))) {
							data.numberOfHardCuts++;
							data.cuts(p+1) = SHOT_CHANGE_HARD_CUT;
						} // if
						removeSoftCut (hist, p-1, p+1, 30);

					} else {
						if (!cutExist (data, left-1, right+1)) {

							data.numberOfDissolves++;
							for (int32 u=left; u<=right; u++) {
								data.cuts(u) = SHOT_CHANGE_DISSOLVE;
							} // for
							removeSoftCut (hist, left, right, 30);
						} // if
					} // else
					
					for (int32 ii=0; ii<size[DIM_X]; ii++) {
						avg2b(ii) = 0.0;		
						ieee64 n=0;
						for (int32 t=5; t<=30; t++) {
							avg2b(ii) += hist[t].values[ii];
							n++;
						} // for
						if (n>0) {
							avg2b(ii) /= n;
						} // else
					} // for
					
				} // if
			} // if
		} // if
	} // for

	// TODO 
	histInitialized[0]=false;		
	initHistogram (0);
	return;
} // detectDissolves


void 
CutDetection::findUnknownSceneChanges (CutDetectionData& data) 
{
	// read histogram values
	initHistogram (0);

	// initialize variables
	data.numberOfUnknownSceneChanges = 0;

	// TODO
	//ieee64 th= 0.5*data.threshold +0.5* data.thHardCut;
	ieee64 th= 0.25*data.threshold +0.75* data.thHardCut;

	for (uint32 j=1; j< hist[0].frames-1; j++) {

		// compare with threshold
		//ieee64 tttemp = hist[0].values[j];
		if (hist[0].values[j] > th) {
			if (!cutExist (data, j+1, j+1)) {

				// mark following frame as cut 
				data.numberOfUnknownSceneChanges++;			
				data.cuts(j+1) = SHOT_CHANGE_OTHER;
			} // if
		} // if
	} // for

	return;
} // detectUnknownSceneChanges 



void 
CutDetection::getDissolveArea (CutDetectionData& cutData, Signal<int32> &dissolvePosition) 
{
	/*
	// read video statistics and summarize values 
	VideoStatisticData vsItem;
	VideoStatistic vs (filename);
	vs.process(vsItem);

	SignalSize size(vsItem.frames);
	Signal<int32> sig (size);
	for (int32 i=0; i<vsItem.frames; i++) {
		sig(i) = (int32) (vsItem.imgData(i).sigma[0] + vsItem.imgData(i).sigma[1] + vsItem.imgData(i).sigma[2]);
	} // for
	*/

	dissolvePosition.resize(sigmaSize);
	for (int32 i=0; i<sigmaSize[DIM_X]; i++) {
		dissolvePosition(i)=0;
	} // for
	const int32 DISSOLVE_AREA   =    1;
	const int32 DISSOLVE_LEFT   =  100;
	const int32 DISSOLVE_CENTER = 1000;
	const int32 DISSOLVE_RIGHT  =10000;



	// (2) analyse values
	for (int32 i=0; i<sigmaSize[DIM_X]; i++) {
		
		// find minimum
		int32 fadeCenter=i;
		int32 n=0;
		while ((fadeCenter>0) && (sigma(fadeCenter-1) <= sigma(fadeCenter))) {
			if (sigma(fadeCenter-1) == sigma(fadeCenter)) {
				n++;
			} else {
				n=0;
			} // else
			fadeCenter--;
		} // while
		fadeCenter+=n/2;
		while ((fadeCenter<sigmaSize[DIM_X]-1) && (sigma(fadeCenter+1) <= sigma(fadeCenter))) {
			if (sigma(fadeCenter+1) == sigma(fadeCenter)) {
				n++;
			} else {
				n=0;
			} // else
			fadeCenter++;
		} // while
		fadeCenter-=n/2;
		
		// find left and right border
		int32 fadeLeft  = fadeCenter;
		int32 fadeRight = fadeCenter;
		int32 fadeStep = -2;
		while ((fadeLeft>0) && (sigma(fadeLeft-1) >= sigma(fadeLeft)+fadeStep/2)) {
			if (fadeStep < sigma(fadeLeft-1) - sigma(fadeLeft)) {
				fadeStep = sigma(fadeLeft-1) - sigma(fadeLeft);
			} // if
			if (fadeStep>10) {
				fadeStep=10;
			} // if
			fadeLeft--;
		} // while
		fadeStep = -2;
		while ((fadeRight<sigmaSize[DIM_X]-1) && (sigma(fadeRight+1) >= sigma(fadeRight)+fadeStep/2)) {
			if (fadeStep < sigma(fadeRight+1) - sigma(fadeRight)) {
				fadeStep = sigma(fadeRight+1) - sigma(fadeRight);
			} // if
			if (fadeStep>10) {
				fadeStep=10;
			} // if
			fadeRight++;
		} // while
	
		//int32 fadeWidth  = fadeRight-fadeLeft;
		//int32 fadeWidth1 = fadeRight-fadeCenter;
		//int32 fadeWidth2 = fadeCenter-fadeLeft;
		int32 fadeHeight1 = sigma(fadeRight)-sigma(fadeCenter);
		int32 fadeHeight2 = sigma(fadeLeft) -sigma(fadeCenter);
		if (sigma(fadeLeft)>sigma(fadeRight)) {
			fadeHeight1 = sigma(fadeLeft) -sigma(fadeCenter);
			fadeHeight2 = sigma(fadeRight)-sigma(fadeCenter);
		} // if
		
		const int32 thFadeHeight  =16;
		const int32 thFadeWidth   = 2;
		const int32 thFadeHeight2 = 2;
		if ((fadeHeight1>thFadeHeight) && 
			(fadeRight-fadeLeft > thFadeWidth) &&
			(fadeHeight2 > thFadeHeight2)) {

			// is there a cut?
			if (!cutExist (cutData, fadeLeft-1, fadeRight+1)) {
				for (int32 tt=fadeLeft; tt<=fadeRight; tt++) {
					if (dissolvePosition(tt) == 0) {
						dissolvePosition(tt)=DISSOLVE_AREA;
					} // if
				} // for
				dissolvePosition(fadeLeft)+=DISSOLVE_LEFT;
				dissolvePosition(fadeCenter)+=DISSOLVE_CENTER;
				dissolvePosition(fadeRight)+=DISSOLVE_RIGHT;
				//} // if
			} // if
		} // if
		i=fadeRight;
		// find maximum
		while ((i<sigmaSize[DIM_X]-1) && (sigma(i) <= sigma(i+1))) {
			i++;
		} // while
		// find minimum
		while ((i<sigmaSize[DIM_X]-1) && (sigma(i) >= sigma(i+1))) {
			i++;
		} // while
	} // for
	return;
} // getDissolveArea


// returns true, if there is any cut between firstFrame and lastFrame
bool 
CutDetection::cutExist (CutDetectionData& data, int32 firstFrame, int32 lastFrame)
{
	bool cut = false;
	if (firstFrame < 0) {
		firstFrame=0;
	} // if
	if (lastFrame >= data.cuts.getSize()[DIM_X]) {
		lastFrame = data.cuts.getSize()[DIM_X] -1;
	} // if
	if (lastFrame <0) {
		lastFrame=0;
	} // if
	for (int32 i=firstFrame; i<=lastFrame; i++) {
		if ((data.cuts(i) == SHOT_CHANGE_HARD_CUT) || 
			(data.cuts(i) == SHOT_CHANGE_DISSOLVE) || 
			(data.cuts(i) == SHOT_CHANGE_DISSOLVE_FIRST) || 
			(data.cuts(i) == SHOT_CHANGE_DISSOLVE_LAST) || 
			(data.cuts(i) == SHOT_CHANGE_FADE_IN) || 
			(data.cuts(i) == SHOT_CHANGE_FADE_OUT) ) {
			cut=true;
		} // if
	} // for

	return cut;
} // cutExist



void 
CutDetection::removeHardCut (HistogramDifferenceItem *item, int32 frame, int32 lastT) 
{
	// delete peak and replace triangle
	for (int32 t=0; t<=lastT; t++) {
		item[t].values[frame-1] = 0.0;
	} // for
	
	// set vertical values
	for (int32 t=1; t<=lastT; t++) {
		for (int32 v=0; v<t; v++) {
			item[t].values[frame-2-v] = item[v].values[frame-2-v];
		} // for				
	} // for
	
	// set diagonal values
	for (int32 t=1; t<=lastT; t++) {
		for (int32 v=0; v<t; v++) {
			if (item[t].values[frame-1-v] < item[t-v-1].values[frame]) {
				item[t].values[frame-1-v] = item[t-v-1].values[frame];
			} // if
		} // for				
	} // for

	return;
} // removeHardCut 



void 
CutDetection::removeSoftCut (HistogramDifferenceItem *item, int32 firstFrame, int32 lastFrame, int32 lastT) 
{
	// delete values
	for (int32 i=firstFrame; i<=lastFrame; i++) {
		for (int32 t=0; t<=lastT; t++) {
			item[t].values[i] = 0.0;
		} // for
	} // for

	// set vertical values
	for (int32 t=1; t<=lastT; t++) {
		for (int32 v=0; v<t; v++) {
			int32 p=firstFrame-1-v;
			if (p>=0) {
				item[t].values[p] = item[v].values[p];
			} // if
		} // for				
	} // for
	
	// set diagonal values
	for (int32 t=1; t<=lastT; t++) {
		for (int32 v=0; v<t; v++) {
			int32 p=lastFrame-v;
			if (p>=0) {
				if (item[t].values[p] < item[t-v-1].values[lastFrame+1]) {
					item[t].values[p] = item[t-v-1].values[lastFrame+1];
				} // if
			} // if
		} // for				
	} // for

	return;
} // removeSoftCut 


void 
CutDetection::readValues (CutDetectionData& data) 
{
	// be sure, that last histogram is calculated first
	initHistogram (31);

	// set variables
	data.version = CUT_DETECTION_VERSION;
	data.numberOfHardCuts = 0;
	data.numberOfFadeIn   = 0;
	data.numberOfFadeOut  = 0;
	data.numberOfDissolves= 0;
	data.numberOfUnknownSceneChanges=0;
	data.threshold = 0.0;
	data.thHardCut=0.0;
	data.numberOfFrames = 0;

	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	data.filename = mocaPath.file;

	// resize signal
	VideoDecoder reader (pathFilename);
	VideoSize vidSize = reader.getSize();
	data.numberOfFrames = vidSize[DIM_Z];
	data.cuts.resize(SignalSize(data.numberOfFrames));
	for (int32 i=0; i<data.numberOfFrames; i++) {
		data.cuts(i) = SHOT_CHANGE_UNKNOWN;
	} // for


	// read data: histograms
	cerr << endl << "initialize values..." << endl;
	for (int32 i=0; i<31; i++) {
		int32 p = (int32)(100.0 * i / 31.0);
		cerr << "\r" << p << " %              " << ends;
		initHistogram (i);
	} // for
	cerr << "\r               " << endl;

    
	
    if (detectFades || detectDissolves) {

        // read data: standard deviation
		VideoStatisticData vsItem;
		VideoStatistic vs (pathFilename);
		vs.process(vsItem);

		sigmaSize[DIM_X]    = vsItem.frames;
		sigmaSize[DIM_Y]    = 1;
		sigmaSize[DIM_BAND] = 1;
		sigmaSize[DIM_Z]    = 1;
		sigma.resize(sigmaSize);

		for (int32 i=0; i<vsItem.frames; i++) {
			sigma(i) = (int32) (vsItem.imgData[i].sigma[0] + 4.0*vsItem.imgData[i].sigma[1] + 4.0*vsItem.imgData[i].sigma[2]);
		} // for

	}
    

	// hard cuts
	cerr << "detect hard cuts..." << endl;
	findHardCuts (data);
	removeFalseHardCuts(data);

    if (detectFades) {
        // fades
        cerr << "detect fades.." << endl;
        findFades (data);
    }

    if (detectDissolves) {
        // short dissolves
        cerr << "detect short dissolves..." << endl;
        findShortDissolves (data);
    
        // dissolves
        cerr << "detect dissolves..." << endl;
        findDissolves (data);

        // other 
        cerr << "other scene changes..." << endl;
        findUnknownSceneChanges (data);
    }
	
	return;	
} // readValues
