/*
   This file is part of the MoCA video library.
   Name: Face.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef FACE_H
#define FACE_H

// C++ includes
#include<iomanip>
#include<iostream>
using namespace std;

// MoCA includes
#include<ImagePosition.h>

const static string FACE_DETECTION_VERSION = "Version-0.61";


class Face {
public:
    friend bool operator==(const Face&, const Face&);
    friend bool operator!=(const Face&, const Face&);
    friend bool operator< (const Face&, const Face&);

    Face& operator=(const Face &other) {
		if (this != &other) {
			frame           = other.frame;
			x               = other.x;
			y               = other.y;
			width           = other.width;
			angle           = other.angle;
			neighbourWeight = other.neighbourWeight;
			weight          = other.weight;
			neighbourCounter= other.neighbourCounter;
		} // if
        return *this;
    } // operator=
	

    int frame;
    int x, y;
    int width;
    int angle;
    double neighbourWeight;
    double weight;
    int neighbourCounter;

    Face() :
            frame(0), x(0), y(0), width(0), angle(0), neighbourWeight(0.0),
            weight(0.0), neighbourCounter(0) { };
    Face(int f, int i, int j, int wi, int a, double we) :
            frame(f), x(i), y(j), width(wi), angle(a), neighbourWeight(0.0),
            weight(we), neighbourCounter(0) { };
    
    bool nextTo(const Face&);

private:
    bool nextTo(const Face&, const int, const int);
};
std::ostream& operator<< (ostream &, const Face&);
std::ostream& operator<< (ostream &, const list<Face>&);



struct FaceItem
{
	FaceItem () 
	{
		clear();
	} 
    FaceItem& operator=(const FaceItem &other) {
		if (this != &other) {
			p1    = other.p1;
			p2    = other.p2;
			angle = other.angle;
			frame = other.frame;
		} // if
        return *this;
    } // operator=
    bool operator< (const FaceItem& other) const {
		return (frame < other.frame);
	}
	void clear()
	{
		angle = 0;
		frame = -1;
		p1[DIM_X]=-1;
		p1[DIM_Y]=-1;
		p2[DIM_X]=-1;
		p2[DIM_Y]=-1;
		return;
	}
	bool isFace ()
	{
		return (p1[DIM_X]>=0 && p1[DIM_Y]>=0 && p2[DIM_X]>=0 && p2[DIM_Y]>=0);
	}
	ImagePosition p1;
	ImagePosition p2;
	int16 angle;
	int32 frame;
};
std::ostream& operator<< (ostream&,    const FaceItem &);
OTagStream&   operator<< (OTagStream&, const FaceItem &);
ITagStream&   operator>> (ITagStream&, FaceItem &);



// list of faces in one mpeg video
struct FaceData
{
	FaceData () 
	{
		clear();
	} 
	~FaceData () 
	{
		faces.clear();
	} 
    FaceData& operator=(const FaceData &other) {
		if (this != &other) {
			version = other.version;
			filename= other.filename;
			numberOfFaces = other.numberOfFaces;
			faces         = other.faces;
		} // if
        return *this;
    } // operator=

	void clear()
	{
		version = FACE_DETECTION_VERSION;
		filename= "";
		numberOfFaces = 0;
		faces.clear();
		return;
	}

	string version;
	string filename;
	int32 numberOfFaces;
	list<FaceItem> faces;
};
std::ostream& operator<< (ostream&,    const FaceData &);
OTagStream&   operator<< (OTagStream&, const FaceData &);
ITagStream&   operator>> (ITagStream&, FaceData &);




#endif





