/*
   This file is part of the MoCA video library.
   Name: ImageStatistic.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef IMAGE_STATISTIC_H
#define IMAGE_STATISTIC_H

// C includes
#include <math.h>

// C++ includes
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <MocaEnum.h>
#include <EdgeDetectionFilter.h>


struct ImageStatisticData
{
	ImageStatisticData () 
	{
		avg [0] = 0;
		avg [1] = 0;
		avg [2] = 0;
		sigma [0] = 0;
		sigma [1] = 0;
		sigma [2] = 0;
		contrastMaxMin    = 0;
		contrastHistogram = 0;
		contrastScale     = 0;
		avgEdgeStrength   = 0;
		color   = TYPE_UNKNOWN;
	} 
	uint8 avg [3];
	int32 sigma [3];
	uint8 contrastMaxMin;
	uint8 contrastHistogram;
	uint8 contrastScale;
	uint8 avgEdgeStrength;
	GrayscaleType color;
};
std::ostream& operator<< (ostream&, const ImageStatisticData&);



// MoCA includes
#include <MocaSignal.h>
#include <Image.h>
#include <HistogramFeature.h>
#include <ScaleFilter.h>
#include <GaussFilter.h>
#include <OImageStream.h>


/*
	Constructor: initializes variables
	
	process(img, data): 
		calculate the image statistics
*/
class ImageStatistic
{
public:

	ImageStatistic() {};
	~ImageStatistic() {};

	// calculate image staistic data (average, sigma, contrast)
	void process (Image<uint8>&, ImageStatisticData&);

	// calculate average color(colors) of image
	void processAvg(Image<uint8>& img, ImageStatisticData& data);

private:

};

#endif
