/*
    ObjectSegmentationFeature.cpp     segment an object
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <ObjectSegmentationFeature.h>


std::ostream& operator<< (ostream& os, const ObjectSegmentationContour& data)
{	
	os << "frame: " << data.frame<< endl;
	int32 sizeX = data.contourParameter.getSize()[DIM_X];
	for (int32 i=0; i<sizeX; i++) {
		os << data.contourParameter(i,0) << " " << data.contourParameter(i,1) << endl;
	} // for

	return os;
} // operator<<

OTagStream&   operator<< (OTagStream& os, const ObjectSegmentationContour& data)
{
	TagFormat::writeStartTag(os, "ObjectSegmentationContour");
	os << endl;
    TagFormat::writeNodeValue(os, "frameNumber", data.frame);
	os << endl;

	TagFormat::writeStartTag(os, "pixelPosition");
	os << endl;
	int32 sizeX = data.contourParameter.getSize()[DIM_X];
	for (int32 i=0; i<sizeX; i++) {
		os << data.contourParameter(i,0) << " " << data.contourParameter(i,1) << " ";
	} // for
	os << endl;
	TagFormat::writeEndTag(os, "pixelPosition");

	TagFormat::writeEndTag(os, "ObjectSegmentationContour");

    return os;
} // operator<<

ITagStream&   operator>> (ITagStream& is, ObjectSegmentationContour& data)
{
    char c;
    bool failFlag = false;
	
	if (is >> c) {
		is.putback(c);

		TagFormat::readStartTag(is, "ObjectSegmentationContour");
		TagFormat::readNodeValue(is, "frameNumber", data.frame);

		TagFormat::readStartTag(is, "pixelPosition");
		int16 v=0;
		int32 sizeX = data.contourParameter.getSize()[DIM_X];
		for (int32 i=0; i<sizeX; i++) {
			is >> v;
			data.contourParameter(i,0) = v;
			is >> v;
			data.contourParameter(i,1) = v;
		} // for
		TagFormat::readEndTag(is, "pixelPosition");

		TagFormat::readEndTag(is, "ObjectSegmentationContour");

	} // if
		
	if (is.rdstate() & ios::failbit) {
		failFlag = true;
	} // if
	if (failFlag) {
		throw MocaException ("ObjectSegmentation: Illegal file format!");
	} else {
		is.clear();
	} // else
	
	return is;
} // operator<<



std::ostream& operator<< (ostream& os, const ObjectSegmentationData& data)
{	
	int32 numberOfFrames = data.data.size();
	os << "numberOfFrames: " << numberOfFrames << endl;
	os << "version       : " << data.version << endl;
	os << "filename      : " << data.filename << endl;

	/*
	list<ObjectSegmentationContour>::const_iterator it;
	for (it=data.data.begin(); it!=data.data.end(); it++) {
		os << (*it);
	} // for
	*/
	os << endl;
	
	return os;
} // operator<<


OTagStream&   operator<< (OTagStream& os, const ObjectSegmentationData& data)
{
	int32 numberOfFrames = data.data.size();
	TagFormat::writeNodeValue(os, "version", data.version);
	TagFormat::writeNodeValue(os, "filename", data.filename);
	TagFormat::writeNodeValue(os, "numberOfFrames", numberOfFrames);
	os << endl;
	
	list<ObjectSegmentationContour>::const_iterator it;
	for (it = data.data.begin(); it != data.data.end(); it++) {
		os << (*it) << endl;
	} // for

	return os;
} // operator<<

ITagStream&   operator>> (ITagStream& is, ObjectSegmentationData& data)
{
	char c;
	bool failFlag = false;
	
	if (is >> c) {
		is.putback(c);
		
		TagFormat::readNodeValue(is, "version", data.version);
		TagFormat::readNodeValue(is, "filename", data.filename);
		TagFormat::readNodeValue(is, "numberOfFrames", data.numberOfFrames);
		if (is.rdstate() & ios::failbit) {
			failFlag = true;
		} // if
		
		data.data.clear();
		ObjectSegmentationContour item;
		for (int32 i=0; i<data.numberOfFrames; i++) {
			if (is >> item) {
				data.data.push_back (item);
			} else {
				failFlag = true;
				break;
			} // else
		} // for
	} // if
	
	if (failFlag) {
		throw MocaException ("ObjectMaskData: Illegal file format!");
	} else {
		is.clear();
	} // else
	
	return is;
}




void 
ObjectSegmentation::process(ObjectSegmentationData &data) 
{

	// read data
	ReadMocaData<ObjectSegmentationData> reader (pathFilename, OBJECT_SEGMENTATION_FILENAME);
	bool dataValid = reader.process (OBJECT_SEGMENTATION_VERSION, data); 

	if (!dataValid) {

		// create new index
		cerr << "ObjectSegmentation: Creating index file ..." << endl;
		readValues(data);
			
		// save index file
		WriteMocaData<ObjectSegmentationData> writer (pathFilename, OBJECT_SEGMENTATION_FILENAME);
		writer.process(data);
		cerr << "\rObjectSegmentation: Creating index file ...done                          " << endl;
	} // if

	return;
} // process


void 
ObjectSegmentation::readValues(ObjectSegmentationData &data) 
{
cerr << "TODO: removed" << endl;
#if 0
	// set variables
	const uint8 backgroundColor = 255;
	const uint8 foregroundColor = 255-backgroundColor;
	const uint8 edgeColor = 255;

	data.version = OBJECT_SEGMENTATION_VERSION;
	data.filename = pathFilename;
	data.numberOfFrames = 0;
	data.data.clear();


	// get object masks
	ObjectMaskData objectMasks;
	{
		ObjectMask objRecognition(pathFilename, cropLeft, cropRight, cropTop, cropBottom);
		objRecognition.process(objectMasks);
		//cerr << objectMasks << endl;
	}

	// get MPEG index
	MpegIndexData index;
	{
		MpegIndex reader (pathFilename);
		reader.process (index);
	}


#if 1
bool tmpLoop=true;
#endif
	// for each object
	list<ObjectMaskItem>::iterator it;
	for (it=data.data.begin(); it!=data.data.end(); it++) {
	
#if 1
		if (tmpLoop) {
			tmpLoop=false;
			it++;
			it++;
			it++;
			it++;
			it++;
			it++;
			it++;
			it++;
			it++;
			it++;
		}
#endif

		// TODO remove
		ObjectMaskItem item = (*it);
		cout << (*it);

		if ((*it).error == 0) {
			
			int32 sizeX = index.videoSize[DIM_X] - cropLeft - cropRight;
			int32 sizeY = index.videoSize[DIM_Y] - cropTop - cropBottom;
			//		IYuvVideo reader ((*it).filenameObjectMaskYuv, sizeX, sizeY, CODEC_Y);
			IYuvVideo reader ((*it).filenameObjectsYuv, sizeX, sizeY, CODEC_Y);
			
			VideoSize size = reader.getSize();
			Image<uint8> imgObjMask;
			for (int32 i=0; i<size[DIM_Z]; i++) {

#if 0
				// debug information
				strstream s;
				s << "-" << setfill('0') << setw(2) << i << "-" << ends;
				string s1 = s.str();
				const string fn01 = "V:/img" + s1 + "-01-mask.pgm";
				const string fn02 = "V:/img" + s1 + "-02-object.pgm";
				const string fn03 = "V:/img" + s1 + "-03-paramObject.pgm";
				const string fn04 = "V:/img" + s1 + "-04-orig.pgm";
				const string fn05 = "V:/img" + s1 + "-05-cropOrig.pgm";
				const string fn06 = "V:/img" + s1 + "-06-cropMask.pgm";
				const string fn07 = "V:/img" + s1 + "-07-cropSmooth.pgm";
				const string fn08 = "V:/img" + s1 + "-08-cropEdge.pgm";
				const string fn09 = "V:/img" + s1 + "-09-cropEdgeBin.pgm";
				const string fn10 = "V:/img" + s1 + "-10-cropEdgeBin2.pgm";
				const string fn11 = "V:/img" + s1 + "-11-cropDilate.pgm";
				const string fn12 = "V:/img" + s1 + "-12-cropNewMask.pgm";
				const string fn13 = "V:/img" + s1 + "-13-cropErose.pgm";
				const string fn14 = "V:/img" + s1 + "-14-cropNewMask2.pgm";
				const string fn15 = "V:/img" + s1 + "-15-maskResult.pgm";
#endif
				
				cerr << "\rfind objects in frame: " << ((*it).firstFrame + i) << ends;

				// read image
				reader.getImage(i,imgObjMask);
				InverseFilter invF;
				invF.process(imgObjMask);
				
			
				// set border of image to background color (only 2 pixel width)
				const int borderWidth = 2;
				for (int32 y=0; y<size[DIM_Y]; y++) {
					for (int32 x=0; x<borderWidth; x++) {
						imgObjMask(x,y) = backgroundColor;
					} // for
					for (int32 x=size[DIM_X]-borderWidth; x<size[DIM_X]; x++) {
						imgObjMask(x,y) = backgroundColor;
					} // for
				} // for
				for (int32 x=0; x<size[DIM_X]; x++) {
					for (int32 y=0; y<borderWidth; y++) {
						imgObjMask(x,y) = backgroundColor;
					} // for
					for (int32 y=size[DIM_Y]-borderWidth; y<size[DIM_Y]; y++) {
						imgObjMask(x,y) = backgroundColor;
					} // for
				} // for
				
#if 0
				WriteImage (fn01, imgObjMask);
#endif
				
				// locate largest object (not background)
				{
					cerr << "TODO: RegionFeature (very slow)" << endl; 
					RegionFeature<uint8> rf;
					Image<uint8> objectImage;
					bool objectFound = false;
					rf.process(imgObjMask, objectImage, objectFound);
					if (!objectFound) continue;

					imgObjMask = objectImage;
				}
				
#if 0
				WriteImage (fn02, imgObjMask);
#endif
				
				
				
				// get, parametrize and save contour
				list<ImageEdgePosition> contourData;
				{
					ContourEdgeList<uint8> contour;
					contour.process (imgObjMask, contourData);
				}
				
				list<ImagePosition> contourParamData;
				{
					ContourParametrization contourParam(ContourParametrization::DEFAULT_CONTOUR_SIZE);
					contourParam.process(contourData, contourParamData);
				}
				
#if 0
				{
					Image<uint8> tmpImg = imgObjMask;
					for (int32 x=0; x<size[DIM_X]; x++) {
						for (int32 y=0; y<size[DIM_Y]; y++) {
							tmpImg(x,y) = Color::WHITE[0];
						} // for
					} // for
					list<ImagePosition>::iterator itParam;
					DrawDot dot;
					for (itParam = contourParamData.begin(); itParam != contourParamData.end(); itParam++) {
						dot.process((*itParam), Color::BLACK[0], tmpImg);
					} // for
					WriteImage (fn03, tmpImg);
				}
#endif
				
				
				
				
				// TODO: optimize contour
				Image<uint8> imgOrig;
				{
					cerr << "TODO: image has object mask, but the mask is not precise, use edges" << endl;
					
					// read original image
					IYuvVideo readerOrigVid ((*it).filenameYuv, sizeX, sizeY, CODEC_Y);
					readerOrigVid.getImage(i,imgOrig);
				}

#if 0
				WriteImage (fn04, imgOrig);	
					
#endif

				// get crop parameters
				ImagePosition cropP1;
				ImagePosition cropP2;
				{
					cropP1[DIM_X] = 30000;
					cropP2[DIM_X] = 0;
					cropP1[DIM_Y] = 30000;
					cropP2[DIM_Y] = 0;
					list<ImagePosition>::iterator itParam;
					for (itParam = contourParamData.begin(); itParam != contourParamData.end(); itParam++) {
						if (cropP1[DIM_X] > (*itParam)[DIM_X]) cropP1[DIM_X] = (*itParam)[DIM_X];
						if (cropP2[DIM_X] < (*itParam)[DIM_X]) cropP2[DIM_X] = (*itParam)[DIM_X];
						if (cropP1[DIM_Y] > (*itParam)[DIM_Y]) cropP1[DIM_Y] = (*itParam)[DIM_Y];
						if (cropP2[DIM_Y] < (*itParam)[DIM_Y]) cropP2[DIM_Y] = (*itParam)[DIM_Y];
					} // for
					if (cropP1[DIM_X]>0) cropP1[DIM_X]--;
					if (cropP1[DIM_Y]>0) cropP1[DIM_Y]--;
					if (cropP2[DIM_X]<index.videoSize[DIM_X]-1) cropP2[DIM_X]++;
					if (cropP2[DIM_Y]<index.videoSize[DIM_Y]-1) cropP2[DIM_Y]++;
				}
				

				// crop images
				Image<uint8> imgCropOrig;
				Image<uint8> imgCropMask;
				{
					CropFilter<uint8> crop (cropP1, cropP2);
					crop.process(imgOrig, imgCropOrig);
					crop.process(imgObjMask, imgCropMask);
				}
#if 0
					WriteImage (fn05, imgCropOrig);	
					WriteImage (fn06, imgCropMask);	
#endif


				// smooth areas around the object mask
				Image<uint8> imgCropSmooth;
				{
					GaussFilter<uint8> gauss (3);
					gauss.process(imgCropOrig, imgCropSmooth);

					// copy original values within mask
					ImageSize cropImgSize = imgCropSmooth.getSize();
					for (int32 y=0; y<cropImgSize[DIM_Y]; y++) {
						for (int32 x=0; x<cropImgSize[DIM_X]; x++) {
							if (imgCropMask(x,y) == foregroundColor) {
								imgCropSmooth (x,y) = imgCropOrig (x,y);
							} // if
						} // for
					} // for
				}
#if 0
				WriteImage (fn07, imgCropSmooth);	
#endif

				// find edges
				Image<uint8> imgCropEdge;
				{
					CannyFilter<uint8> canny;
					canny.process(imgCropSmooth, imgCropEdge);
				}
#if 0
				WriteImage (fn08, imgCropEdge);	
#endif
					
				// binarize edge image
				Image<uint8> imgCropEdgeBin;
				{
					BinarizeFilter<uint8> bf (128);
					bf.process(imgCropEdge, imgCropEdgeBin);
				}
#if 0
				WriteImage (fn09, imgCropEdgeBin);	
#endif

				// remove edges outside the object
				{
					ImageSize cropImgSize = imgCropEdgeBin.getSize();
					for (int32 y=0; y<cropImgSize[DIM_Y]; y++) {
						for (int32 x=0; x<cropImgSize[DIM_X]; x++) {
							if (imgCropMask(x,y) == backgroundColor) {
								imgCropEdgeBin (x,y) = 255 - edgeColor;
							} // if
						} // for
					} // for
				}
#if 0
				WriteImage (fn10, imgCropEdgeBin);	
#endif


				// find nearest edge at the borders of the object mask
				Image<uint8> imgCropDilate;
				Image<uint8> imgCropErose;
				Image<uint8> imgNewMask;
				Image<uint8> imgNewMask2;
				Image<uint8> imgTemp;
				imgTemp = imgCropMask;
				for (int32 i=0; i<10; i++)
				{
					// erose the object
					bool invertColors=true;
					ErosionFilter<uint8> erose (5, invertColors);
					erose.process (imgTemp, imgCropErose);

					imgNewMask = imgCropErose;
					ImageSize cropImgSize = imgCropEdgeBin.getSize();
					for (int32 y=0; y<cropImgSize[DIM_Y]; y++) {
						for (int32 x=0; x<cropImgSize[DIM_X]; x++) {
							if (imgCropEdgeBin(x,y) == edgeColor) {
								imgNewMask (x,y) = foregroundColor;
							} // if
						} // for
					} // for

					// dilate the object
					DilationFilter<uint8> dilate (3, invertColors);
					dilate.process (imgNewMask, imgCropDilate);

					// remove pixel outside the mask
					imgNewMask2 = imgCropDilate;
					for (int32 y=0; y<cropImgSize[DIM_Y]; y++) {
						for (int32 x=0; x<cropImgSize[DIM_X]; x++) {
							if (imgCropMask(x,y) == backgroundColor) {
								imgNewMask2 (x,y) = backgroundColor;
							} // if
						} // for
					} // for

					imgTemp = imgNewMask2;

#if 0
					WriteImage (fn11, imgCropDilate);	
					WriteImage (fn12, imgNewMask);	
					WriteImage (fn13, imgCropErose);	
					WriteImage (fn14, imgNewMask2);	
					WriteImage (fn15, imgTemp);	
#endif
				}
				
				// TODO 
				
				// save data
				{
					ObjectSegmentationContour item;
					item.frame = i;
					list<ImagePosition>::iterator itParam;
					int32 n=0;
					for (itParam = contourParamData.begin(); itParam != contourParamData.end(); itParam++) {
						item.contourParameter(n,0) = (*itParam)[DIM_X];
						item.contourParameter(n,1) = (*itParam)[DIM_Y];
						n++;
					} // for
					data.numberOfFrames++;
					data.data.push_back(item);
				}

			} // if
		} // for

//cerr << "DEBUG: REMOVE break;" << endl;
//break;
	} // for
#endif
	return;
} // process

