/*
   This file is part of the MoCA video library.
   Name: BinarizeFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef BINARIZE_FILTER_H
#define BINARIZE_FILTER_H

// MoCA includes
#include <SampleBlock.h>
#include <GetType.h>
#include <FilterBase.h>
#include <InverseFilter.h>


//! thresholds image
template<class T>
class BinarizeFilter : public FilterBase<T>
{
public:
    //! Constructor with a single threshold for each band
    BinarizeFilter(T);

    //! Constructor for a different threshold for each band
    BinarizeFilter(T *, sizeType);

    //! Constructor for different thresholds and only certain bands
    BinarizeFilter(T *, sizeType, bool *);

    //! Constructor for thresholds for the first three bands
    BinarizeFilter(T, T, T, bool, bool, bool);

    //! Destructor
    ~BinarizeFilter();

    //! Apply Binarize to image
    void process(SampleBlock<T> &srcDest);

    //! Apply Binarize to image
    void process(const SampleBlock<T> &src, SampleBlock<T> &dest);

protected:

private:

    void filterLine(const T* srcPixel, const offsetType* srcOffsetPixel,
                    T* destPixel, const offsetType* destOffsetPixel,
                    sizeType size, T threshold);

    //! the threshold
    T threshold;

    //! array holding thresholds, one for each band
    T* thresholdArray;

    //! size of thresholdArray
    sizeType sizeArray;

    //! array holding information which band should be thresholded
    bool* applyArray;

}; // class BinarizeFilter

#include <BinarizeFilter.cpt>

#endif
