/*
   This file is part of the MoCA video library.
   Name: CannyEdgeDetectionFilter.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: CannyEdgeDetectionFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $

*/

#ifndef CANNY_EDGE_DETECTION_H
#define CANNY_EDGE_DETECTION_H


#include <ImageFilterBase.h>

/*!

  \brief Canny filter

  This filter is based on the implementation of the canny edge
  detection filter of the iue (Image Understanding Environment)

  Original Source:

  Author(s): John Canny.
  Title: A Computational Approach to Edge Detection.
  Published: IEEE Transactions on Pattern Analysis and Machine Intelligence 8 (6), pp. 579--698, 1986.

  \todo the implemenation is not complete, albeit usable

*/
template<class T>
class CannyEdgeDetectionFilter : public ImageFilterBase<T>
{
public:
    //! the default constructor
    CannyEdgeDetectionFilter(bool verbose = false);
    //! the destructor
    ~CannyEdgeDetectionFilter();

    //! the filter itself
    void process(Image<T>& srcDest)
    {
	Image<uint8> tmp=srcDest;
	process(tmp, srcDest);
	return;
    }
    void process(const Image<T>& src, Image<T>& dest);

private:
    void Smooth_image(const Image<T>& src, int, float, Image<ieee64>& dest);
    void Non_maximal_supression(const Image<ieee64>& grad,
                                Image<ieee64>& dx, // in and out
                                Image<ieee64>& dy, // in and out
                                int width,
                                float low,
                                Image<ieee64>& thick, // in and out
                                Image<ieee64>& theta // in and out
                                );
    void Initial_hysteresis(Image<ieee64>& thin, const float high, const float low, const int width);
    void Initial_follow(Image<ieee64> &thin,
                        float low,
                        int x, int y,
                        list<int> &xc,
                        list<int> &yc,
                        list<float> &grad);
    void Thin_edges(Image<ieee64> &thin, const float low, const int width);
    /*    void Find_junctions(const Image<ieee64> &thin,
                        const int width,
                        const float low,
                        list<int> &xjunc,
                        list<int> &yjunc,
                        Image<int> &junction);

    void Find_junction_clusters(Image<int> &junctions, const int width);
    void Cluster_centre_of_gravity(Image<ieee64> &src,
                                   Image<int> &jx,
                                   Image<int> &jy,
                                   list<int>& xc,
                                   list<int> &yc,
                                   int &x0,
                                   int &y0);
    */
    bool verbose;               // print out debug messages?

}; // CannyEdgeDetectionFilter

struct Edgel {
    int x;
    int y;
    float grad;
}; // Edgel


#include <CannyEdgeDetectionFilter.cpt>

#endif
