/*
   This file is part of the MoCA video library.
   Name: DenoiseFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef DENOISE_FILTER_H
#define DENOISE_FILTER_H

// C includes
#include <math.h>

// MoCA includes
#include <ImageFilterBase.h>
#include <SampleBlock.h>
#include <GetType.h>



/*!

  The filter itself, provide an input sample block, the data in the
  input sample block gets overwritten with the output data.

  This method calls filter(src, dest)

  \sa DenoiseFilter<T>::filter(SampleBlock<T>, SampleBlock<T>)

  \param srcDest input and output sample block

 */



//! denoise image
template<class T>
class DenoiseFilter: public ImageFilterBase<T>
{
public:
    //! Constructor with no arguments
    DenoiseFilter();

    //! Constructor with relative theshold 
    DenoiseFilter(double threshold1, double threshold2);

    //! Destructor
    ~DenoiseFilter(){};

    //! Apply Denoise to image
    void process(Image<T> &srcDest)
	{
		Image<T> tmp=srcDest;
		process(tmp, srcDest);
		return;
	}

    //! Apply Denoise to image
    void process(const Image<T> &src, Image<T> &dest);

protected:

private:

    //! threshold1  the surrounding pixels must be similar
    T threshold1; //  
    //! threshold2  the central pixel must be different
    T threshold2;


}; // class DenoiseFilter

#include <DenoiseFilter.cpt>

#endif
