/*
    DifferenceFilter.cpp     get difference of two images
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <DifferenceFilter.h>


void
DifferenceFilter::process(const SampleBlock<uint8> &img1, const SampleBlock<uint8> &img2, SampleBlock<uint8> &diff) 
{
	diff = img2;
	process (img1, diff);
	return;
} // process


void
DifferenceFilter::process(const SampleBlock<uint8> &img1, SampleBlock<uint8> &srcDiff) 
{
    SampleBlockSize size  = img1.getSize();
    SampleBlockSize size2 = srcDiff.getSize();

    if ((size[DIM_X] != size2[DIM_X]) || (size[DIM_Y] != size2[DIM_Y]) || (size[DIM_BAND] != size2[DIM_BAND])) {
        throw MocaException("Imagesizes do not match!");
    } // if

	
    for (indexType band = 0; band < size[DIM_BAND]; band++) {
        for (indexType z = 0; z < size[DIM_Z]; z++) {
			for (indexType y = 0; y < size[DIM_Y]; y++) {
				
				// set offset
				const uint8* img1Ptr    = img1.getSamplePtr (0, y, z, band);
				uint8* srcDiffPtr = srcDiff.getSamplePtr (0, y, z, band);

				const offsetType* offsetPtrImg1    = &(img1.getSampleOffsetX (0));
				const offsetType* offsetPtrSrcDiff = &(srcDiff.getSampleOffsetX (0));
				
				for (indexType x = 0; x < size[DIM_X]; x++) {
					
					*srcDiffPtr = (256 + (*img1Ptr) - (*srcDiffPtr)) / 2;
					
					img1Ptr += *offsetPtrImg1;
					srcDiffPtr += *offsetPtrSrcDiff;
					
					offsetPtrImg1 += 1;
					offsetPtrSrcDiff += 1;
				} // for
			} // for
		} // for
	} // for
	return;
} // process



#if 0
template<class T>
void
DifferenceFilter<T>::process(const Image<T> &one, const Image<T> &two, Image<T> &diff) {
    ImageSize oneSize = one.getSize();
    ImageSize twoSize = two.getSize();

    if (oneSize[DIM_X] != twoSize[DIM_X]
        || oneSize[DIM_Y] != twoSize[DIM_Y]
        || oneSize[DIM_BAND] != twoSize[DIM_BAND]) {
        // throw an exception!
        throw MocaException("Imagesizes do not match!");
    } // if

    diff.resize(oneSize);

    T maxT=(T)0;
    maxT = GetType::getMax(maxT);

    for (int x = 0; x < oneSize[DIM_X]; x++) {
        for (int y = 0; y < oneSize[DIM_Y]; y++) {
            for (int band = 0; band < oneSize[DIM_BAND]; band++) {
                T minDiff = maxT;
                for (int widthX = -(masksize / 2); widthX <= masksize / 2; widthX++) {
                    for (int widthY = -(masksize / 2); widthY <= masksize / 2; widthY++) {
                        T tempDiff = (T)fabs(one(x, y, band) - two(x + widthX, y + widthY , band));
                        if (tempDiff < minDiff) {
                            minDiff = tempDiff;
                        } // if
                    } // for
                } // for
                diff(x, y, band) = minDiff;
            } // for
        } // for
    } // for

} // process
#endif

