/*
   This file is part of the MoCA video library.
   Name: EdgeDetectionFilter.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: EdgeDetectionFilter.h,v 1.2 2004/03/02 17:04:11 kopf Exp $

*/

#ifndef EDGE_DETECTION_FILTER_H
#define EDGE_DETECTION_FILTER_H

#include <cmath>

#include <Image.h>
#include <ImageFilterBase.h>
#include <GetType.h>
#include <BinarizeFilter.h>


/*!
  various edge detection filters combined in one
*/
template<class T>
class EdgeDetectionFilter: public ImageFilterBase<T>
{
public:
    //! the default constructor
    EdgeDetectionFilter(ieee64 th=-1);

    //! the destructor
    ~EdgeDetectionFilter();

    void process(Image<T>& srcDest) {
		Image<T> tmp=srcDest;
        process(tmp, srcDest);
		return;
    } // process

    void process(const Image<T>& src, Image<T>& dest) {
        filterSobel(src, dest);
		return;
    } // process

	// count sum of edges
    void countEdgeValue (const Image<T>& src, ieee64 &value);


    void filterRoberts(const Image<T>& src, Image<T>& dest);

    void filterSobel(const Image<T>& src, Image<T>& dest);

    void filterPrewitt(const Image<T>& src, Image<T>& dest);

    void filterKirsch(const Image<T>& src, Image<T>& dest);

    void filterSogmc(const Image<T>& src, Image<T>& dest);

    void filterClaudia(const Image<T>& src, Image<T>& dest);

    void filterRobinson(const Image<T>& src, Image<T>& dest);

    void filterMarrHildreth(const Image<T>& src, Image<T>& dest);

private:
	void applyTh (Image<T>& src)
	{
		if (th != -1) {
		  T v = (T)th;
		  BinarizeFilter<T> bf(v);
		  bf.process (src);
		} // if
		return;
	}

	ieee64 th;
}; // EdgeDetectionFilter


#include<EdgeDetectionFilter.cpt>

#endif
