/*
   This file is part of the MoCA video library.
   Name: EdgeOrientationFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: EdgeFilter.h,v 1.2 2004/03/02 17:04:11 kopf Exp $

*/

#ifndef EDGE_ORIENTATION_FILTER_H
#define EDGE_ORIENTATION_FILTER_H

// C includes
#include <math.h>

// MoCA includes
#include <Image.h>
#include <ImageFilterBase.h>

/*!

  \brief Class to filter the edges and the orientation of the edges of a given image

  This class implements an edge finding filter. Use this class to
  filter the edges of the image and its orientation. The image itself will be changed to
  the edge filtered image.
  "dest" stored the edge strength (band=0) and its orientation (band=1).

*/
template <class T>
class EdgeOrientationFilter: public ImageFilterBase<T>
{
public:

	/*!
    This constructor takes up tp 2 parameters
  
	\param th is a threshold of type int32 above which a found "edge" will be marked as one
	\sa process()
	*/
    EdgeOrientationFilter(int32 th = 60)
		:th(th) {}

    //! Default destructor
    ~EdgeOrientationFilter(){}

    //! Filtering the edges of the image
    void process (Image<T>& srcDest)
	{
		Image<T> tmp = srcDest;
		process(tmp, srcDest);
		return;
	}

	// the destination image stores:
	//  band=0: edge strength (that is zero if edge is below threshold th)
	//  band=1: edge orientation (0-180 degrees, 181: undefined orientation)
	void process (const Image<T>& src, Image<T>& dest);

private:


	int32 th;
}; //class EdgeOrientationFilter

#include <EdgeOrientationFilter.cpt>

#endif
