/*
   This file is part of the MoCA video library.
   Name: GrayscaleFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef GRAYSCALE_FILTER_H
#define GRAYSCALE_FILTER_H

#include <ImageFilterBase.h>
#include <Image.h>
#include <GetType.h>

/*
Y601_GRAYSCALE_FILTER		calculate luma image from RGB image according 
							to Rec. Y601  (0.299 * r + 0.587 * g + 0.114 * b)
Y709_GRAYSCALE_FILTER		calculate the theoretical correct luma image from RGB image 
							according to Rec. Y709   (0.212 * r + 0.701 * g + 0.087 * b)
Y601_GRAYSCALE_FILTER_MOD   int (0.299 * r + 0.587 * g + 0.114 * b)
SYMMETRIC_GRAYSCALE_FILTER	(r+g+b) / 3
*/
enum {Y601_GRAYSCALE_FILTER = 0, Y709_GRAYSCALE_FILTER = 1, Y601_GRAYSCALE_FILTER_MOD = 2, SYMMETRIC_GRAYSCALE_FILTER = 3};


//! thresholds image
template<class T>
class GrayscaleFilter : public ImageFilterBase<T>
{
public:
    //! Constructor 
    GrayscaleFilter(): method(Y601_GRAYSCALE_FILTER){};

    //! Constructor 
    GrayscaleFilter(int method): method(method){};

    //! Destructor
    ~GrayscaleFilter() {};

    //! convert image to grayscale image
    void process(Image<T> &srcDest);

    //! convert image to grayscale image
    void process(const Image<T> &src, Image<T> &dest);

protected:
private:
	int method;

}; // class GrayscaleFilter

#include <GrayscaleFilter.cpt>

#endif
