/*
   This file is part of the MoCA video library.
   Name: LightingCorrection.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef LIGHTING_CORRECTION_H
#define LIGHTING_CORRECTION_H


#include <Image.h>
#include <LuminanceFilter.h>


template<class T> 
class LightingCorrection{
public:
	// constructor
	LightingCorrection(){
		image = NULL;
		columnSum = NULL;
		rowSum = NULL;
		lightingCorrectionAllocatedSize=-1;
		lightingCorrectionX=-1;
		lightingCorrectionX2=-1;
	};
	
	//destructor
	~LightingCorrection(){};
	
	 /* if only src image as parameter
	  * @param srcDest image holding the src image
	  */
	 void process(Image<T> &srcDest);
	
	/*! the filter
	 * @param src image containing the src image
	 * @param dest image getting the processed image
	 */
    void process(const Image<T> &src, Image<T> &dest);
    
private:
	void copyFromImage(const Image<uint8> &img);
	void copyToImage(ImageSize &is, Image<uint8> &img);
	
	int **image;
	
	int *columnSum;
    int *rowSum;
	
	int lightingCorrectionAllocatedSize;
    int lightingCorrectionX;
    int lightingCorrectionX2;
};

#include <LightingCorrection.cpt>

#endif
