/*
   This file is part of the MoCA video library.
   Name: LucasKanadeFilter.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   Functions for block motion vector estimation

   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: LucasKanadeFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $

*/

#ifndef LUCASKANADEFILTER_H
#define LUCASKANADEFILTER_H

// C++ includes
#include <vector>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <Video.h>
#include <GaussFilter.h>
#include <DownSampleFilter.h>
#include <OImageStream.h>


/*!
  Filter for calculating optical flow using the Lucas Kanade algorithm.
  The current implementation is iterative and pyramid-based

  See: Bruce D. Lucas, Takeo Kanade.
       An Iterative Image Registration Technique with an Application to Stereo Vision.
       Proceedings 7th International Conference on Artificial Intelligence, 674-679, 1981.

  \todo replace array of fixed size (weight[64][64])
  \todo replace fixed number of pyramid layers
*/

template<class T>
class LucasKanadeFilter 
{
public:
    //! The constructor
    LucasKanadeFilter(sizeType maskSize, ieee64 noiseLevel)
            : maskSize(maskSize),
              noiseLevel(noiseLevel) {}

    //! The destructor
    ~LucasKanadeFilter() {}

    //! Calculate motion vectors
    void process (const Video<T>&, Video<ieee64>&);

private:
    void lucasKanadeStep(indexType, indexType, indexType, VideoVector<T> &, ieee64&, ieee64&);
    void calculateReliability(const Video<T>&, indexType, Image<ieee64>&);

    sizeType maskSize;
    ieee64 noiseLevel;
    ieee64 weight[64][64]; // Gaussian window, ATTENTION: HACK !!!
}; // LucasKanadeFilter


#include <LucasKanadeFilter.cpt>

#endif

