/*
   This file is part of the MoCA video library.
   Name: MarkovFilter.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: MarkovFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $

*/

#ifndef MARKOV_FILTER_H
#define MARKOV_FILTER_H

// C-includes
#include <math.h>

// MoCA-includes
#include <Image.h>
#include <OImageStream.h>
#include <OpeningFilter.h>


/*!

  \brief calculate the difference of two images 

  This filter takes two images and calculates the difference for each
  pixel, writing it into the third image.
  
 */
class MarkovFilter
{
public:
    //! the default constructor
    MarkovFilter() {};

    //! destructor
    ~MarkovFilter() {};

    //! the filter
    void process(const Image<uint8> &lumi, const Image<uint8> &diff, Image<uint8> &dest);

    //! the filter
    void process(const Image<uint8> &lumi, Image<uint8> &diffDest)
	{
		Image<uint8> tmp = diffDest;
		process(lumi, tmp, diffDest);
		return;
	} // process

private:

	inline double ProbBack(int32 v,int32 busy)
	{
		double x=v-128;
		double b=busy/255.0;
		double fact = 0.6+b/3;
		
		// 20
		x /= 30.0;  // taxi: 7, normal: 10
		x*=x;
		return fact*exp(-x);
	}



}; // MarkovFilter

#endif
