/*
    DrawIcon.cpp     draw an icon 
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include<DrawIcon.h>


void 
DrawIcon::process (const ImagePosition &p, IconType icon, Image<uint8>& srcDest)
{

    // pixel position (x,y) for one icon (upper left corner)
    const int SIZE_CAR = 2*59;
    const int32 CAR[SIZE_CAR] = {3,16,3,17,3,18,3,19,3,20,4,16,4,20,5,16,5,20,5,21,6,16,6,19,6,22,7,15,7,
		         16,7,18,7,23,8,14,8,18,8,23,9,12,9,13,9,19,9,22,10,12,10,20,10,21,11,12,11,20,12,12,12,
				 20,13,12,13,20,14,12,14,20,15,13,15,20,15,21,16,14,16,19,16,22,17,15,17,16,17,18,17,23,
				 18,16,18,18,18,23,19,16,19,19,19,22,20,16,20,20,20,21,21,16,21,17,21,18,21,19,21,20};
    const int SIZE_BOAT = 2*68;
    const int32 BOAT[SIZE_BOAT] = {0,21,0,22,1,21,1,23,2,21,2,24,3,21,3,25,4,21,4,25,5,21,5,25,6,21,6,25,
		           7,21,7,25,8,21,8,25,9,8,9,9,9,10,9,11,9,12,9,13,9,14,9,15,9,16,9,17,9,18,9,19,9,20,9,
				   21,9,25,10,10,10,18,10,21,10,25,11,11,11,18,11,21,11,25,12,12,12,18,12,21,12,25,13,13,
				   13,18,13,21,13,25,14,14,14,18,14,21,14,25,15,15,15,18,15,21,15,25,16,16,16,18,16,21,
				   16,24,17,17,17,18,17,21,17,23,18,18,18,21,18,22};
    const int SIZE_PEOPLE = 2*52;
    const int32 PEOPLE[SIZE_PEOPLE] = {23,21,24,22,24,36,24,37,25,16,25,17,25,18,25,23,25,34,25,35,26,15,
		               26,19,26,24,26,32,26,33,27,14,27,20,27,25,27,30,27,31,28,14,28,20,28,21,28,22,28,
					   23,28,24,28,25,28,26,28,27,28,28,28,29,28,30,29,14,29,20,29,25,29,30,29,31,30,15,
					   30,19,30,24,30,32,30,33,31,16,31,17,31,18,31,23,31,34,31,35,32,22,32,36,32,37,33,
					   21};
    
    ImageSize size = srcDest.getSize();
    int32 numberOfPixel = 0;
    const int32 *ptr = NULL;
    switch (icon) {
	case ICON_CAR:
	    numberOfPixel = SIZE_CAR;
	    ptr = &CAR[0];
	    break;
	case ICON_BOAT:
	    numberOfPixel = SIZE_BOAT;
	    ptr = &BOAT[0];
	    break;
	case ICON_PEOPLE:
	    numberOfPixel = SIZE_PEOPLE;
	    ptr = &PEOPLE[0];
	    break;
	default: 
	    cerr << "DrawIcon: unknown icon!" << endl;
    } // switch

    int32 x = p[DIM_X];
    int32 y = p[DIM_Y];
    for (int32 i=0; i<numberOfPixel; i+=2) {
		int32 px = x+ptr[i];
		int32 py = y+ptr[i+1];
		if ((px>=0) && (py>=0) && (px<size[DIM_X]) && (py<size[DIM_Y])) {
			for (int32 b=0; b<size[DIM_BAND]; b++) {
				srcDest(px,py,b) = c[b];
			} // for
		} // if
    } // for

    return;
} // drawIcon 




// read file, get icon pixels (black), store pixels
void 
DrawIcon::createIcon (const string &filename)
{
    Image<uint8> img;
    ReadImage (filename, img);
    ImageSize size = img.getSize();

    // find first black pixel
    int32 minX=0;
    int32 minY=0;
    for (int32 x=0; x<size[DIM_X]; x++) {
	for (int32 y=0; y<size[DIM_Y]; y++) {
	    if (img(x,y) == 0) {
		minY = y;
		x=10000;
		y=10000;
		break;
	    } // if
	} // for
    } // for
    for (int32 y=0; y<size[DIM_Y]; y++) {
	for (int32 x=0; x<size[DIM_X]; x++) {
	    if (img(x,y) == 0) {
		minX = x;
		x=10000;
		y=10000;
		break;
	    } // if
	} // for
    } // for

    cout << endl;
    int32 n=0;
    for (int32 x=0; x<size[DIM_X]; x++) {
	for (int32 y=0; y<size[DIM_Y]; y++) {
	    if (img(x,y) == 0) {
		cout << x << "," << y << ",";
		n++;
	    } // if
	} // for
    } // for
    cout << endl << endl;
    cout << n << endl << endl;
    exit(1);

} // createIcon


