/*
   This file is part of the MoCA video library.
   Name: MocaGraphics.h
   Copyright (C) 2006 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef MOCA_GRAPHICS_H
#define MOCA_GRAPHICS_H

//
// just include this file, if you have to include many different graphic routines (headers) in your file
//
// include all header files in directory "graphics"
#include <Color.h>
#include <ConvertColorSpace.h>
#include <DrawCircle.h>
#include <DrawCross.h>
#include <DrawDot.h>
#include <DrawIcon.h>
#include <DrawLine.h>
#include <DrawRectangle.h>

void testGraphics ()
{
	// define path with test images
	string libMocaPath="../";  // linux path
#ifdef WIN32
	libMocaPath="../../../../libmoca/";  // windows path
#endif
	string fn=libMocaPath + "doc/lena.jpg";

	Image<uint8> img0, img1;
	ReadImage (fn, img0);
	img1 = img0;

	// test class Color
	Color c = Color::WHITE;

	// test class ConvertColorSpace
	VideoCodec srcMode=CODEC_RGB;
	VideoCodec destMode=CODEC_YUV444;
	ConvertColorSpace::process (srcMode, destMode, img1, img0);

	// test class DrawCircle
	ImagePosition p0 (20,20);
	int32 r=10;
	DrawCircle dci;
	dci.process (p0, r, img0);

	// test class DrawCross
	DrawCross dc;
	dc.process(p0, img0);

	// test class DrawDot
	ImagePosition p1 (70,70);
	DrawDot dd;
	dd.process(p0, img0);

	// test class DrawIcon
	p0[DIM_X]+=20;
	p0[DIM_Y]+=20;
	IconType icon=ICON_CAR;
	DrawIcon di (c);
	di.process(p0, icon, img0);

	// test class DrawLine
	p1[DIM_X]-=10;
	p1[DIM_Y]-=50;
	DrawLine dl;
	dl.process(p0, p1, img0);

	// test class DrawRectangle
	DrawRectangle dr;
	dr.process(p0, p1, img0);

	// store test image
	fn=libMocaPath + "doc/lena-test-graphics.jpg";
	WriteImage (img0, fn);
	cerr << "file saved in YUV color space (not RGB color space): " << fn << endl;
	return;
}

#endif

