/*
   This file is part of the MoCA video library.
   Name: AudioDecoder.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

  AudioDecoder.h -- Interface for MPEG/AVI audio input
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef AUDIO_DECODER_H
#define AUDIO_DECODER_H

// C++ includes
#include <fstream>
#include <vector>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <MocaSignal.h>
#include <WriteMocaData.h>
#include <VideoIndex.h>
#include <OWaveAudio.h>
#include <OpenProcess.h>


const static string AUDIO_DECODER_VERSION    = "Version-0.1";
const static string AUDIO_DECODER_FILENAME   = "audiodecoder.idx";

struct AudioDecoderData
{
	AudioDecoderData () 
	{
		version = AUDIO_DECODER_VERSION;
		filename = "";
		audioFilename = "";
		hasAudio = false;
		audioData.setHeader (0, 0, 0, 0);

	} 
    AudioDecoderData& operator=(const AudioDecoderData &other) {
		if (this != &other) {
			version       = other.version;
			hasAudio      = other.hasAudio;
			filename      = other.filename;
			audioFilename = other.audioFilename;
			audioData     = other.audioData;
		} // if
        return *this;
    } // operator=

	string version;
	bool hasAudio;         
	string filename;       // MPEG/AVI filename
	string audioFilename;  // stored WAV filename
	WaveHeader audioData;  // audio parameters (channels, ...)
};

std::ostream& operator<< (ostream&,    const AudioDecoderData&);
OTagStream&   operator<< (OTagStream&, const AudioDecoderData&);
ITagStream&   operator>> (ITagStream&, AudioDecoderData&);


// this class opens a video (MPEG/AVI) and stores the audio track as WAV-file
// the parameters of the audio are stored in AudioDecoderData
class AudioDecoder 
{
public:

	// constuctor defines filename of mpeg audio/video file
	AudioDecoder (string &pathFilename);

	// destructor
	~AudioDecoder ()
	{
		close();
	}
	
	void process (AudioDecoderData &data);
	string getAudioPathFilename()
	{
		return audioPathFilename;
	}

private:
	void open(string &cmd);
	void close();
	void readValues (AudioDecoderData &data);

	FILE* fh_in;
	string pathFilename;
	string audioPathFilename;
	string videodecodingExecutable;

	VideoIndexData index;
};

#endif
