/*
   This file is part of the MoCA video library.
   Name: OJpegFormatStream.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: OJpegFormatStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef O_JPEG_FORMAT_STREAM_H
#define O_JPEG_FORMAT_STREAM_H

// MoCA includes
#include <MocaTypes.h>

#ifdef HAVE_LIBJPEG

// jpeg includes
extern "C" {
#include <jpeglib.h>
#include <jerror.h>
}

// MoCA includes
#include <GetType.h>
#include <OImageFormatStream.h>
#include <JpegParameter.h>


/*!
  jpeg format specific stream
*/
template<class T>
class OJpegFormatStream : public OImageFormatStream<T>
{
public:
    //! the default constructor
    OJpegFormatStream(ostream &os, const FormatParameter *fp = NULL) : OImageFormatStream<T>(os), pp(NULL) { open (os, fp); }

    //! the destructor
    ~OJpegFormatStream() { if (pp != NULL) delete pp; }

    //! open format stream
    void open(ostream &, const FormatParameter *);

    //! write an image
    void write(const Image<T> &) throw (MocaException);

private:
    //! local jpeg parameter pointer copy thingy
    JpegParameter* pp;

  //! user defined error handling
  struct my_error_mgr {
    struct jpeg_error_mgr pub;	/* "public" fields */

    jmp_buf setjmp_buffer;	/* for return to caller */
  };

  typedef struct my_error_mgr * my_error_ptr;

  //! data destination manager stuff
  typedef struct {
    struct jpeg_destination_mgr pub; /* public fields */

    ostream* outfile;		/* target stream */
    JOCTET * buffer;		/* start of buffer */
  } my_destination_mgr;

  typedef my_destination_mgr * my_dest_ptr;

  enum { OUTPUT_BUF_SIZE = 4096 };
  static void init_destination_ostream (j_compress_ptr);
  static boolean empty_output_buffer_ostream (j_compress_ptr);
  static void term_destination_ostream (j_compress_ptr);
  static void jpeg_ostream_dest (j_compress_ptr, ostream*);
  static void my_error_exit (j_common_ptr cinfo);
}; // OJpegFormatStream

#include <OJpegFormatStream.cpt>

#endif

#endif


