/*
   This file is part of the MoCA video library.
   Name: OpenFile.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   Color.h  -- defines a struct Color and several default colors

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: {kuehne,kopf}@pi4.informatik.uni-mannheim.de

   $Id: 
*/



#ifndef OPEN_FILE_H
#define OPEN_FILE_H

// C++ includes
#include <stdio.h>

// C++ includes
#include <string>
#include <iostream>
using namespace std;



enum OpenFileMode {
    F_UNKNOWN,
    F_READ,
    F_WRITE
};

class OpenFile 
{
public:

    // open file
    //   d_fh: file handler
    //   cmd:  command
    //   mode: 1=read, 2=write 
    
    static void process(FILE **d_fh, string &filename, OpenFileMode mode=F_UNKNOWN, bool msg=true) 
    {

	// set flags
	string fileOpenFlags = "";
	switch (mode) {
	    case F_READ:
		fileOpenFlags = "r";
#ifdef _WIN32
		fileOpenFlags = "rb";
#endif
		break;
	    case F_WRITE:
		fileOpenFlags = "w";
#ifdef _WIN32
		fileOpenFlags = "wb";
#endif
		break;
	    default:
		cerr << "OpenFile::unknown mode, please use P_READ or P_WRITE." << endl;
		cerr << "             cannot open file: " << filename << endl;
		exit(1);
		break;
	} // switch

	// print
	if (msg) {
		cerr << "open file: (" << fileOpenFlags << ")  (" << filename << ") " << endl; 
	} // if

	// open file
	(*d_fh)=NULL;
	(*d_fh) = fopen(filename.c_str(),fileOpenFlags.c_str());

	// is error?
	if ((*d_fh) == NULL) {
	    cerr << "OpenFile::cannot open file: " << filename << "    (fh==NULL)" << endl;
	    exit(1);
	} // if

	// is error?
	int code = ferror((*d_fh));
	if (code != 0) {
	    cerr << "OpenFile::cannot open file: " << filename << "    (ferror)" << endl;
	    exit(1);
	} // if

	// is error?
	code = feof((*d_fh));
	if (code != 0) {
	    cerr << "OpenFile::cannot open file: " << filename << "    (ferror)" << endl;
	    exit(1);
	} // if
	
	return;
    } // file

};

#endif

