/*
   This file is part of the MoCA video library.
   Name: VideoEncoderBase.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

  VideoEncoderBase.h -- Abstract class to encode videos
                        derived classes: Avi, Yuv, Ppm, Pgm
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef VIDEO_ENCODER_BASE_H
#define VIDEO_ENCODER_BASE_H

// C++ includes
#include <string>
using namespace std;

// MoCA includes
#include <Image.h>
#include <Video.h>


// this class provides the basic routines to encode a video
class VideoEncoderBase
{
protected:

	// constuctor defines filename 
	VideoEncoderBase ();
	VideoEncoderBase (const string &pathFile);

	// destructor
	~VideoEncoderBase ();
	

	// abstract classes
	virtual void write (Image<uint8> &img) = 0;
	virtual void write (Video<uint8> &vid) = 0;
	virtual void write (ImageVector<uint8> &vid) = 0;

protected:

	virtual void init () = 0;
	void open ();
	void close ();


	// file name and index
	string pathFilename;

	// file handler
	ofstream *os;
};


#endif
