/*
   This file is part of the MoCA video library.
   Name: Gexel.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#ifndef GEXEL_H
#define GEXEL_H

// C includes
#include <math.h>

// C++ includes
#include <vector>

// MoCA includes
#include <MocaExceptions.h>

/*!

  \brief General element

  Base class for Sixel, Pixel and Voxel

*/
template<class T>
class Gexel
{
public:
    //! the default constructor
    Gexel(SampleBlockPosition &pos) : pos(pos) {};

    //! the default constructor
    Gexel(indexType x, indexType y = 0, indexType z = 0) : pos(pos) {};

    //! the copy constructor
    Gexel(const Gexel<T> &g) : values(g.values), pos(g.getPos())  {};

    //! the assignment operator
    Gexel &operator=(const Gexel &g)
    {
        pos = g.getPos();
        values = g.values;
    } // operator=

    //! the destructor
    virtual ~Gexel() {};

    //! Values of the Gexel over all bands
    vector<T> values;

    //! getPosition
    const SampleBlockPosition &getPos() const
    {
        return pos;
    } // getPos

    //! set position
    void setPos(SampleBlockPosition &newPos)
    {
        pos = newPos;
    } // setPos

    //! distance of two pixels based on the values
    T distance(Gexel<T> &g)
    {
        if (values.size() != g.values.size()) {
            throw new MocaException
                ("Number of bands do not match!");
        } // if

        double temp = 0;
        for (int i = 0; i < values.size(); i++) {
            temp += values[i] * g.values[i];
        } // for

        return (T) sqrt(temp);
    } // distance

protected:

private:
    //! Position of the Gexel in a SampleBlock
    SampleBlockPosition pos;

}; // Gexel

#endif

