/*
   This file is part of the MoCA video library.
   Name: ImageEdgePosition.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   file.h  -- short description

   Author:    Gerald Kuehne und Oliver der Held
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kuehne@pi4.informatik.uni-mannheim.de

   $Id: ImageEdgePosition.h,v 1.1 2004/02/17 12:00:58 kopf Exp $
*/


#ifndef IMAGE_EDGE_POSITION_H
#define IMAGE_EDGE_POSITION_H

// C++ includes
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaEnum.h>
#include <ImagePosition.h>

class ImageEdgePosition : public ImagePosition
{
public:
    ImageEdgePosition () : ImagePosition () {}
    ImageEdgePosition (indexType x, indexType y, indexType band, EdgePosition epos)
            : ImagePosition (x, y, band), edgePosition (epos) {}
    ImageEdgePosition (indexType x, indexType y, EdgePosition epos)
            : ImagePosition (x, y, 0), edgePosition (epos) {}

    EdgePosition getEdgePosition () const { return edgePosition; }

    bool operator== (const ImageEdgePosition&) const;
    bool operator!= (const ImageEdgePosition&) const;

    friend std::ostream& operator<< (ostream&, const ImageEdgePosition&);

protected:
    EdgePosition edgePosition;
};

inline
bool
ImageEdgePosition::
operator== (const ImageEdgePosition& other) const
{
  return(position[DIM_X]    == other.position[DIM_X] &&
         position[DIM_Y]    == other.position[DIM_Y] &&
         edgePosition       == other.edgePosition);
} // operator==

inline
bool
ImageEdgePosition::
operator!= (const ImageEdgePosition& other) const
{
  return !(*this == other);
} // operator!=


#endif
