/*
   This file is part of the MoCA video library.
   Name: MocaEnum.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef ENUM_H
#define ENUM_H

// Moca include
#include <MocaTypes.h>
#include <MocaMacros.h>

enum ImageFileType {
	UNKNOWN_FILE = 0,
	JPG_FILE     = 1,
	PNG_FILE     = 2,
	PGM_FILE     = 3,
	PNM_FILE     = 4
};

enum BorderMode {
	BORDER_REPLICATE,
	BORDER_REFLECTING,
	BORDER_PERIODIC
};

enum Dimensions {
	DIM_X = 0,
	DIM_Y = 1,
	DIM_Z = 2,
	DIM_BAND = 3,
	DIM_N = 4
};

enum Directions {
    DIR_X   = 1,
    DIR_Y   = 2,
    DIR_XY  = 3,
    DIR_Z   = 4,
    DIR_XZ  = 5,
    DIR_YZ  = 6,
    DIR_XYZ = 7
};

enum EdgePosition {
    NORTH,
    SOUTH,
    EAST,
    WEST
};

enum Neighbours {
    N4,		// 4 direct neighbours
    N8,		// 8 neighbours
    Nd,		
    Nm
};


//video codecs:
//  CODEC_RGB   : usual formal of images in the MoCA library
//  CODEC_Y     : grayscale images (only Y plane)
//  CODEC_YUV444: YUV image with full resolution (DIM_X * DIM_Y) in each color plane (Y, U and V)
//  CODEC_YUV422: YUV image with full resolution (DIM_X * DIM_Y) in Y-plane (luminance)
//                and half horizontal resolution (DIM_X/2 * DIM_Y) in each color plane (U and V)
//  CODEC_YUV420: YUV image with full resolution (DIM_X * DIM_Y) in Y-plane (luminance)
//                and half resolution (DIM_X/2 * DIM_Y/2) in each color plane (U and V)
//  CODEC_YUV8  : instead of three planes in YUV images, only one plane ist used,
//                   bits 0-3: most significant bits of Y plane (least significant bits)
//                   bits 4-5: most significant bits of V plane 
//                   bits 6-7: most significant bits of U plane 
//                do not use this mode, if you want to display an image (the quality
//                is very low, but some algorithms [like region growing] work very well)
enum VideoCodec {
  CODEC_Y         = 1,  // Image<uint8> with size[DIM_BAND]==1
  CODEC_YUV444    = 2,  // Image<uint8> with size[DIM_BAND]==3
  CODEC_YUV422    = 3,  // Image<uint8> with size[DIM_BAND]==3
  CODEC_YUV420    = 4,  // Image<uint8> with size[DIM_BAND]==3
  CODEC_RGB       = 5,  // Image<uint8> with size[DIM_BAND]==3
  CODEC_YUV8      = 6   // Image<uint8> with size[DIM_BAND]==1
};

enum GrayscaleType {
	TYPE_UNKNOWN        = 0, 
	TYPE_COLOR          = 1, 
	TYPE_GRAYSCALE      = 2, 
	TYPE_NEAR_GRAYSCALE = 3
};

// default position of the moving object
enum DefaultObjectPosition {
    POSITION_UNKNOWN = 1,
    POSITION_CENTER  = 2,
    POSITION_LEFT    = 3,
    POSITION_RIGHT   = 4,
    POSITION_TOP     = 5,
    POSITION_BOTTOM  = 6
};

const ieee64 MOCA_PI = 3.1415926535897932;

enum StorageType { 
  UNKNOWN_STORAGE_TYPE = 1, 
  RAW_STORAGE_TYPE     = 2, 
  ASCII_STORAGE_TYPE   = 3
};

#endif




